/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 1002 $
 * $Date: 2020-04-10 23:31:24 +0200 (Fri, 10 Apr 2020) $
 */
package at.lcm.bu21.general.dtypes.editor2.TDataModel;

import at.lcm.bu21.general.dtypes.TBoolean;
import at.lcm.bu21.general.dtypes.TData;

/**
 * Representation of a TBooleanModel - type.
 * 
 * @author Johannes Kaisinger
 */
public class TBooleanModel extends TDataModel {

	/** Data variable */
	private TBoolean data;

	/**
	 * Create object with initial value false
	 * 
	 */
	public TBooleanModel() {
		this.data = new TBoolean(false);
	}

	/**
	 * Create object with given data (TBoolean)
	 * 
	 * @param data
	 *            Reference of data (TBoolean)
	 * 
	 */
	public TBooleanModel(TBoolean data) {
		this.data = data;
	}

	/**
	 * Get data (TBoolean)
	 * 
	 * @return Data Reference of data (TBoolean)
	 */
	public Boolean getValue() {
		return this.data.getBoolean();
	}

	/**
	 * Get data (TData)
	 * 
	 * @return data Reference of data (TData)
	 */
	@Override
	public TData getData() {
		return this.data;
	}

	/**
	 * Set data
	 * 
	 * @param value
	 *            New value of data
	 */
	public void setValue(boolean value) {
		TBoolean tboolean = new TBoolean(value);
		this.data = tboolean;
	}

}
