/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 1002 $
 * $Date: 2020-04-10 23:31:24 +0200 (Fri, 10 Apr 2020) $
 */
package at.lcm.bu21.general.dtypes.editor2;

import javax.swing.table.AbstractTableModel;

import at.lcm.bu21.general.dtypes.editor2.TDataModel.TNumericModel;

/**
 * This is a table model for a JTable to view TNumerics. It based on
 * AbstractTableModel.
 * 
 * @author Johannes Kaisinger
 */
public class TNumericTableModel extends AbstractTableModel {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/**
	 * Local reference of the tNumericModel. It allows to manipulate the
	 * TNumeric.
	 */
	private TNumericModel tNumericModel;

	/**
	 * Creates an object
	 * 
	 * @param tNumericModel
	 *            The TNumericModel
	 */
	public TNumericTableModel(TNumericModel tNumericModel) {
		this.tNumericModel = tNumericModel;
	}

	/**
	 * Creates an object
	 * 
	 * @param column
	 *            Index of column.
	 * @return name The name of the column
	 */
	@Override
	public String getColumnName(int column) {
		return "" + column;
	}

	/**
	 * Get row count of tNumericModel.
	 * 
	 * @return rowCount
	 */
	@Override
	public int getRowCount() {
		return this.tNumericModel.getTNumeric().size()[0];
	}

	/**
	 * Get column count of tNumericModel.
	 * 
	 * @return columnCount
	 */
	@Override
	public int getColumnCount() {
		return this.tNumericModel.getTNumeric().size()[1];
	}

	/**
	 * Get column class.
	 * 
	 * @param column
	 *            Index of column.
	 * @return Class
	 */
	@Override
	public Class<String> getColumnClass(int column) {
		return String.class;
		// return this.getValueAt(0, column).getClass();
	}

	/**
	 * Get if a cell is editable.
	 * 
	 * @param row
	 *            Index of row.
	 * @param column
	 *            Index of column.
	 * @return isEditable
	 */
	@Override
	public boolean isCellEditable(int row, int column) {
		return true;
	}

	/**
	 * Get a cell value.
	 * 
	 * @param row
	 *            Index of row.
	 * @param column
	 *            Index of column.
	 * @return value Value of cell (String).
	 */
	@Override
	public String getValueAt(int row, int column) {
		String value = this.tNumericModel.getValue(row, column);
		return value;
	}

	/**
	 * Get a cell value.
	 *
	 * @param value
	 *            Value of cell (Object)
	 * @param row
	 *            Index of row.
	 * @param column
	 *            Index of column.
	 */
	@Override
	public void setValueAt(Object value, int row, int column) {
		try {
			this.tNumericModel.setValue(row, column, value.toString());
			this.fireTableDataChanged();
		} catch (NumberFormatException e) {
		} catch (StringIndexOutOfBoundsException e) {
		}
	}
}
