#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.5

$LastChangedRevision: 1024 $
$LastChangedDate:: 2020-06-25 18:45:41 +0200#$

This file is part of SyMSpace.

Copyright (c) 2017, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""

if __name__ == "__main__":
    import os
    import nose
    import optparse
    import pkgutil

    py_dir = os.path.dirname(os.path.dirname(__file__))
    start_dir_default = os.path.join(py_dir, 'tests')
    reports_dir_default = os.path.join(py_dir, 'tmp')
    cov_dir_default = os.path.join(py_dir, 'src')

    parser = optparse.OptionParser("usage: %prog [options]")
    parser.add_option("-d", "--startDir", dest="start_dir", default=start_dir_default,
                      type="string", help="The search for tests starts from this directory.")
    parser.add_option("-r", "--reportsDir", dest="reports_dir", default=reports_dir_default,
                      type="string", help="The test reports will be saved to this directory.")
    parser.add_option("-c", "--coverPkgDir", dest="cov_dir", default=cov_dir_default,
                      type="string", help="The packages importable from this directory will be covered.")

    (options, args) = parser.parse_args()
    start_dir = os.path.abspath(options.start_dir)
    reports_dir = os.path.abspath(options.reports_dir)
    cov_dir = os.path.abspath(options.cov_dir)
    cover_xml = True
    cover_html = False

    if start_dir == "":
        raise ValueError("--startDir must be given")
    if reports_dir == "":
        raise ValueError("--reportsDir must be given")
    if cov_dir == "":
        raise ValueError("--coverPkgDir must be given")

    xunit_file = os.path.join(reports_dir, 'nosetests-junit.xml')
    cov_file = os.path.join(reports_dir, 'coverage.xml')

    if not os.path.isdir(reports_dir):
        os.makedirs(reports_dir, exist_ok=True)

    cov_pkgs = [pkgname for _, pkgname, _ in pkgutil.iter_modules([cov_dir])]

    arguments = ['', '--where={}'.format(start_dir),
                 '--with-xunit', '--xunit-file={}'.format(xunit_file),
                 '--with-coverage'
                 ]
    if cover_xml:
        arguments.extend(['--cover-xml',
                          '--cover-xml-file={}'.format(os.path.join(reports_dir, 'coverage.xml')),
                          ])

    if cover_html:
        arguments.extend(['--cover-html',
                          '--cover-html-dir={}'.format(os.path.join(reports_dir, 'html'))
                          ])

    for pkgs in cov_pkgs:
        arguments.append('--cover-package={}'.format(pkgs))

    print('Start Path: {}'.format(start_dir))
    print('Report Path: {}'.format(reports_dir))
    print('Coverage: {}'.format(cov_pkgs))

    nose.run(argv=arguments)
