/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9814 $
 * $Date: 2020-04-30 16:26:03 +0200 (Thu, 30 Apr 2020) $
 */
package at.lcm.symspace.center;

import java.awt.Component;
import java.awt.Desktop;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.rmi.RemoteException;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.filechooser.FileNameExtensionFilter;

import at.lcm.bu21.general.dtypes.TData;
import at.lcm.bu21.general.utils.CloneException;
import at.lcm.bu21.general.utils.FileUtils;
import at.lcm.bu21.magopt.docu.FillExcelReport;
import at.lcm.bu21.magopt.docu.FillTexReport;
import at.lcm.bu21.magopt.docu.ReportExcel;
import at.lcm.bu21.magopt.docu.ReportStructure;
import at.lcm.bu21.magopt.docu.ReportStructureDialog;
import at.lcm.bu21.magopt.structure.IFunction;
import at.lcm.bu21.magopt.structure.IParamContainer;
import at.lcm.bu21.magopt.structure.IParameter;
import at.lcm.bu21.magopt.structure.IProject;
import at.lcm.bu21.magopt.structure.ParamComboBox;
import at.lcm.bu21.magopt.structure.ParamContainer;
import at.lcm.bu21.magopt.structure.ParamDirectory;
import at.lcm.bu21.magopt.structure.ParamField;
import at.lcm.bu21.magopt.structure.ParamFile;
import at.lcm.bu21.magopt.structure.ParamFormula;
import at.lcm.bu21.magopt.structure.ParamFunction;
import at.lcm.bu21.magopt.structure.ParamJavaFunction;
import at.lcm.bu21.magopt.structure.ParamMaterial;
import at.lcm.bu21.magopt.structure.ParamMatlabFunction;
import at.lcm.bu21.magopt.structure.ParamPythonFunction;
import at.lcm.bu21.magopt.structure.Parameter;
import at.lcm.bu21.magopt.structure.Project;
import at.lcm.bu21.magopt.structure.jep.JepProxy;
import at.lcm.bu21.magopt.structure.simulate.SimulateAll;
import at.lcm.bu21.magopt.utils.MagOptConfig;
import at.lcm.bu21.magopt.utils.MatlabControl;
import at.lcm.symspace.Launcher;
import at.lcm.symspace.center.icons.sym.SyMIcons;
import at.lcm.symspace.center.utils.AnalysisDialog;
import at.lcm.symspace.center.utils.MacroDialog;
import at.lcm.symspace.center.utils.VariantsDialog;
import at.lcm.symspace.center.utils.simulate.SimulateAllGui;
import at.lcm.symspace.center.views.CenterStatusBar;
import at.lcm.symspace.center.views.CenterViewManager;
import at.lcm.symspace.gui.About;
import at.lcm.symspace.gui.Controller;
import at.lcm.symspace.gui.utils.Storage;
import at.lcm.symspace.utils.SmartComponentLocation;
import jobmanager.JobDesc;
import magopt.gui.compare.CompareProject;
import magopt.gui.component.browser.ComponentBrowserTreeView;
import magopt.gui.console.PythonConsole;
import magopt.gui.helpers.GetFromDialog;
import magopt.gui.helpers.JLibraryChooser;
import magopt.gui.helpers.JProjectChooser;
import magopt.gui.helpers.SelectUserDialog;
import magopt.gui.helpers.SendToDialog;
import magopt.structure.database.FileConnection;
import magopt.utils.ComponentInfo;
import magopt.utils.Execute;
import magopt.utils.MopFile;
import matlabcontrol.MatlabProxy;

public class CenterController extends Controller {

	private boolean dirty = false;

	private IProject project = null;

	private IParameter selected = null;

	private boolean startSimulation = false;

	private boolean remoteCompute = false;

	private boolean optimze = false;

	public CenterController(CenterGui gui) {
		super(gui);
	}

	@Override
	protected void showProgressBar(final String init) {
		this.showProgressBar(init, 0);
	}

	@Override
	protected void showProgressBar(final String init, final int max) {
		try {
			SwingUtilities.invokeAndWait(new Runnable() {
				@Override
				public void run() {
					CenterController.this.getViewManager().setEnabled(false);
				}
			});
		} catch (InvocationTargetException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		((CenterStatusBar) this.getViewManager().getStatusBar()).showProgressBar(init, max);
	}

	@Override
	protected void updateProgressBar(final int val, final String status) {
		((CenterStatusBar) this.getViewManager().getStatusBar()).updateProgressBar(val, status);
	}

	@Override
	protected void hideProgressBar() {
		((CenterStatusBar) this.getViewManager().getStatusBar()).hideProgressBar();

		try {
			SwingUtilities.invokeAndWait(new Runnable() {
				@Override
				public void run() {
					CenterController.this.getViewManager().setEnabled(true);
				}
			});
		} catch (InvocationTargetException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void openProject(Project proj) throws RemoteException {
		// FIXME IProject
		CenterController.this.project = proj;
		CenterController.this.selected = CenterController.this.project;
		CenterController.this.dirty = false;

		new Storage().addRecentProject(proj.getMopFile().getAbsolutePath());

		CenterController.this.refreshViews();

		// update script direcetory
		CenterViewManager views = this.getViewManager();
		views.pythonConsole.setScriptDirectory(this.project.getMopFile().getParentFile());
	}

	public static void openProjectExtern(File file) {
		Execute exec;
		File java;

		java = new File(new File(System.getProperty("java.home"), "bin"), "java");

		exec = new Execute(java.getPath(), "-cp",
				FileUtils.getFile(MagOptConfig.getHome(), "System", "Java", "magopt.jar").getAbsolutePath(),
				Launcher.class.getName(), "SyMSpaceCenterGui", file.getAbsolutePath());
		exec.setRedirectStreams(true);
		exec.start();

		// CenterGui gui = new CenterGui();
		// gui.getController().mnu_file_open(file);
		// gui.setVisible(true);
	}

	public IProject getProject() {
		return this.project;
	}

	public IParameter getSelected() {
		return this.selected;
	}

	public void setSelected(IParameter param) {
		if (this.selected != param) {
			this.selected = param;
			this.refreshViews();
		}
	}

	public boolean isStartSimulation() {
		return this.startSimulation;
	}

	public boolean isRemoteCompute() {
		return this.remoteCompute;
	}

	public boolean isOptimize() {
		return this.optimze;
	}

	private File getWorkDir() {
		List<String> recentFiles;
		File workDir = null;

		if (this.project != null) {
			try {
				workDir = this.project.getMopFile().getParentFile();
			} catch (RemoteException e) {
				System.err.println(e.getMessage());
				e.printStackTrace();
			}
		}
		if (workDir == null) {
			recentFiles = new Storage().getRecentProjects();
			if ((recentFiles != null) && !recentFiles.isEmpty()) {
				workDir = new File(recentFiles.get(0)).getParentFile();
			}
		}

		return workDir;
	}

	public void mnu_file_new() {
		JFileChooser chooser;

		chooser = new JProjectChooser(JProjectChooser.ALL, new File("."));
		File workDir = this.getWorkDir();
		if (workDir != null) {
			chooser.setCurrentDirectory(workDir);
		}
		if (chooser.showSaveDialog(this.getGuiComponent()) == JFileChooser.APPROVE_OPTION) {
			this.mnu_file_new("ProjectName", "ProjectDesc", chooser.getSelectedFile());
		}
	}

	public void mnu_file_new(final String name, final String description, final File file) {
		new Thread() {
			@Override
			public void run() {
				Project proj;

				CenterController.this.showProgressBar("Create project");

				try {
					proj = new Project(name, description, file);
					if (CenterController.this.project != null) {
						// open in new vm
						proj.save();
						CenterController.openProjectExtern(proj.getMopFile());
					} else {
						// open in this vm
						CenterController.this.openProject(proj);
					}
				} catch (Exception e) {
					System.err.println(e.getMessage());
					e.printStackTrace();
				}

				// CenterController.this.refreshViews();
				CenterController.this.hideProgressBar();
			}
		}.start();

	}

	public void mnu_file_open() {
		JProjectChooser chooser = new JProjectChooser(JProjectChooser.ALL, new File("."));
		File workDir = this.getWorkDir();
		if (workDir != null) {
			chooser.setCurrentDirectory(workDir);
		}
		if (chooser.showOpenDialog(this.getGuiComponent()) == JFileChooser.APPROVE_OPTION) {
			this.mnu_file_open(chooser.getSelectedFile());
		}
	}

	public void mnu_file_open(final File file) {
		new Thread() {
			@Override
			public void run() {
				CenterController.this.showProgressBar("Load project");

				try {
					if (CenterController.this.project != null) {
						// open in new vm
						CenterController.openProjectExtern(file);
					} else {
						// open in new vm
						CenterController.this.openProject(Project.load(file));
					}
				} catch (Exception e) {
					System.err.println(e.getMessage());
					e.printStackTrace();
				}

				// CenterController.this.refreshViews();
				CenterController.this.hideProgressBar();
			}
		}.start();
	}

	public void mnu_file_close() {
		new Thread() {
			@Override
			public void run() {
				boolean close = true;
				final boolean save;
				CenterGui gui;

				if (CenterController.this.project != null) {
					switch (JOptionPane.showConfirmDialog(CenterController.this.getGuiComponent(),
							"Save current project?", "Close Project", JOptionPane.YES_NO_CANCEL_OPTION)) {
					case JOptionPane.YES_OPTION:
						save = true;
						break;
					case JOptionPane.NO_OPTION:
						save = false;
						break;
					default:
						close = false;
						save = false;
					}
				} else {
					save = false;
				}

				if (close) {
					CenterController.this.showProgressBar("Close project");

					if (save) {
						try {
							CenterController.this.project.save();
							// MagOptController.this.dirty = false;
						} catch (Exception e) {
							// FIXME
							System.err.println(e.getMessage());
							e.printStackTrace();
						}

						// MagOptController.this.refreshViews();
					}

					try {
						CenterController.this.project.close();
					} catch (RemoteException e) {
						System.err.println(e.getMessage());
						e.printStackTrace();
					}
					CenterController.this.project = null;
					CenterController.this.selected = null;
					CenterController.this.dirty = false;

					// update script directory
					CenterViewManager views = CenterController.this.getViewManager();
					views.pythonConsole.setScriptDirectory(null);

					CenterController.this.refreshViews();
					CenterController.this.hideProgressBar();

					gui = (CenterGui) CenterController.this.getGuiComponent();
					if (gui != null) {
						gui.close();
					}
				}
			}
		}.start();
	}

	public void mnu_file_save() {
		new Thread() {
			@Override
			public void run() {
				CenterController.this.showProgressBar("Save project");

				try {
					CenterController.this.project.save();
					CenterController.this.dirty = false;
				} catch (Exception e) {
					// FIXME
					System.err.println(e.getMessage());
					e.printStackTrace();
				}

				CenterController.this.refreshViews();
				CenterController.this.hideProgressBar();
			}
		}.start();
	}

	public void mnu_file_save_as() {
		JFileChooser chooser = new JProjectChooser(JProjectChooser.ALL, new File("."));
		File workDir = this.getWorkDir();
		if (workDir != null) {
			chooser.setCurrentDirectory(workDir);
		}
		if (chooser.showSaveDialog(this.getGuiComponent()) == JFileChooser.APPROVE_OPTION) {
			this.mnu_file_save_as(chooser.getSelectedFile());
		}
	}

	public void mnu_file_save_as(final File file) {
		new Thread() {
			@Override
			public void run() {
				CenterController.this.showProgressBar("Save as project");

				try {
					CenterController.this.project.saveAs(file);
					CenterController.this.dirty = false;
				} catch (Exception e) {
					// FIXME
					System.err.println(e.getMessage());
					e.printStackTrace();
				}

				// update script directory
				CenterViewManager views = CenterController.this.getViewManager();
				views.pythonConsole.setScriptDirectory(file.getParentFile());

				new Storage().addRecentProject(file.getAbsolutePath());

				CenterController.this.refreshViews();
				CenterController.this.hideProgressBar();
			}
		}.start();
	}

	public void mnu_file_save_as_template() {
		JFileChooser chooser = new JProjectChooser(JProjectChooser.TEMPLATE_ONLY, new File("."));
		File workDir = this.getWorkDir();
		if (workDir != null) {
			chooser.setCurrentDirectory(workDir);
		}
		if (chooser.showSaveDialog(this.getGuiComponent()) == JFileChooser.APPROVE_OPTION) {
			this.mnu_file_save_as_template(chooser.getSelectedFile());
		}

	}

	public void mnu_file_save_as_template(final File file) {
		new Thread() {
			@Override
			public void run() {
				CenterController.this.showProgressBar("Save template");

				try {
					// FIXME IProject
					// FIXME MultiSelect
					((Project) CenterController.this.project).exportTemplate(file,
							new String[] { CenterController.this.selected.getIdentifier() });
				} catch (Exception e) {
					// FIXME
					System.err.println(e.getMessage());
					e.printStackTrace();
				}

				CenterController.this.refreshViews();
				CenterController.this.hideProgressBar();
			}
		}.start();
	}

	public void mnu_file_exit() {
		boolean exit = true;
		final boolean save;

		if (this.project != null) {
			switch (JOptionPane.showConfirmDialog(this.getGuiComponent(), "Save current project?", "Exit",
					JOptionPane.YES_NO_CANCEL_OPTION)) {
			case JOptionPane.YES_OPTION:
				save = true;
				break;
			case JOptionPane.NO_OPTION:
				save = false;
				break;
			default:
				exit = false;
				save = false;
			}
		} else {
			save = false;
		}

		if (exit) {
			new Thread() {
				@Override
				public void run() {
					CenterGui gui;

					if (save) {
						CenterController.this.showProgressBar("Save project");

						try {
							CenterController.this.project.save();
							CenterController.this.dirty = false;
						} catch (Exception e) {
							// FIXME
							System.err.println(e.getMessage());
							e.printStackTrace();
						}

						CenterController.this.refreshViews();
						CenterController.this.hideProgressBar();
					}

					gui = (CenterGui) CenterController.this.getGuiComponent();
					if (gui != null) {
						gui.exit();
					}
				}
			}.start();
		}
	}

	public void mnu_edit_insert_component() {
		new Thread() {
			@Override
			public void run() {
				ComponentBrowserTreeView view;
				ComponentInfo info;
				ParamContainer parent;
				Project comp;

				parent = CenterController.this.getParent();
				if (parent != null) {
					CenterController.this.showProgressBar("Open component browser");

					view = new ComponentBrowserTreeView(ComponentInfo.listComponents(),
							(JFrame) CenterController.this.getGuiComponent());

					CenterController.this.hideProgressBar();

					info = view.getSelectedItem();
					if (info != null) {
						CenterController.this.showProgressBar("Insert component");

						try {
							comp = Project.load(info.getMop());
							if (view.isSmartInsert()) {
								// TODO IProject
								parent = SmartComponentLocation
										.getSmartContainer((Project) CenterController.this.getProject(), comp);
							}
							// TODO IProject
							((Project) CenterController.this.project).importTemplate(comp, parent);
						} catch (Exception e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}

						CenterController.this.dirty = true;
						CenterController.this.refreshViews();
						CenterController.this.hideProgressBar();
					}
				}
			}
		}.start();
	}

	public void mnu_edit_insert_container() {
		new Thread() {
			@Override
			public void run() {
				String name;
				ParamContainer parent;

				parent = CenterController.this.getParent();
				if (parent != null) {
					name = (String) JOptionPane.showInputDialog(CenterController.this.getGuiComponent(), "Name:",
							"Edit insert container", JOptionPane.QUESTION_MESSAGE, null, null, "Default");
					if ((name != null) && !name.isEmpty()) {
						CenterController.this.showProgressBar("Insert container");

						try {
							// FIXME IParamContainer
							parent.addParam(new ParamContainer(name));
							CenterController.this.dirty = true;
						} catch (Exception e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}

						CenterController.this.refreshViews();
						CenterController.this.hideProgressBar();
					}
				}
			}
		}.start();
	}

	public void mnu_edit_insert_field() {
		new Thread() {
			@Override
			public void run() {
				String name;
				ParamContainer parent;

				parent = CenterController.this.getParent();
				if (parent != null) {
					name = (String) JOptionPane.showInputDialog(CenterController.this.getGuiComponent(), "Name:",
							"Edit insert field", JOptionPane.QUESTION_MESSAGE, null, null, "Default");
					if ((name != null) && !name.isEmpty()) {
						CenterController.this.showProgressBar("Insert field");

						try {
							// FIXME IParamField
							parent.addParam(new ParamField(name));
							CenterController.this.dirty = true;
						} catch (Exception e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}

						CenterController.this.refreshViews();
						CenterController.this.hideProgressBar();
					}
				}
			}
		}.start();
	}

	public void mnu_edit_insert_combobox() {
		new Thread() {
			@Override
			public void run() {
				String name;
				ParamContainer parent;

				parent = CenterController.this.getParent();
				if (parent != null) {
					name = (String) JOptionPane.showInputDialog(CenterController.this.getGuiComponent(), "Name:",
							"Edit insert combobox", JOptionPane.QUESTION_MESSAGE, null, null, "Default");
					if ((name != null) && !name.isEmpty()) {
						CenterController.this.showProgressBar("Insert combobox");

						try {
							// FIXME IParamComboBox
							parent.addParam(new ParamComboBox(name));
							CenterController.this.dirty = true;
						} catch (Exception e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}

						CenterController.this.refreshViews();
						CenterController.this.hideProgressBar();
					}
				}
			}
		}.start();
	}

	public void mnu_edit_insert_formula() {
		new Thread() {
			@Override
			public void run() {
				String name;
				ParamContainer parent;

				parent = CenterController.this.getParent();
				if (parent != null) {
					name = (String) JOptionPane.showInputDialog(CenterController.this.getGuiComponent(), "Name:",
							"Edit insert formula", JOptionPane.QUESTION_MESSAGE, null, null, "Default");
					if ((name != null) && !name.isEmpty()) {
						CenterController.this.showProgressBar("Insert formula");

						try {
							// FIXME IParamFormula
							parent.addParam(new ParamFormula(name));
							CenterController.this.dirty = true;
						} catch (Exception e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}

						CenterController.this.refreshViews();
						CenterController.this.hideProgressBar();
					}
				}
			}
		}.start();
	}

	public void mnu_edit_insert_directory() {
		new Thread() {
			@Override
			public void run() {
				String name;
				ParamContainer parent;

				parent = CenterController.this.getParent();
				if (parent != null) {
					name = (String) JOptionPane.showInputDialog(CenterController.this.getGuiComponent(), "Name:",
							"Edit insert directory", JOptionPane.QUESTION_MESSAGE, null, null, "Default");
					if ((name != null) && !name.isEmpty()) {
						CenterController.this.showProgressBar("Insert directory");

						try {
							// FIXME IParamDirectory
							parent.addParam(new ParamDirectory(name));
							CenterController.this.dirty = true;
						} catch (Exception e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}

						CenterController.this.refreshViews();
						CenterController.this.hideProgressBar();
					}
				}
			}
		}.start();
	}

	public void mnu_edit_insert_file() {
		new Thread() {
			@Override
			public void run() {
				String name;
				ParamContainer parent;

				parent = CenterController.this.getParent();
				if (parent != null) {
					name = (String) JOptionPane.showInputDialog(CenterController.this.getGuiComponent(), "Name:",
							"Edit insert file", JOptionPane.QUESTION_MESSAGE, null, null, "Default");
					if ((name != null) && !name.isEmpty()) {
						CenterController.this.showProgressBar("Insert file");

						try {
							// FIXME IParamFile
							parent.addParam(new ParamFile(name));
							CenterController.this.dirty = true;
						} catch (Exception e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}

						CenterController.this.refreshViews();
						CenterController.this.hideProgressBar();
					}
				}
			}
		}.start();
	}

	public void mnu_edit_insert_material() {
		new Thread() {
			@Override
			public void run() {
				String name;
				ParamContainer parent;

				parent = CenterController.this.getParent();
				if (parent != null) {
					name = (String) JOptionPane.showInputDialog(CenterController.this.getGuiComponent(), "Name:",
							"Edit insert material", JOptionPane.QUESTION_MESSAGE, null, null, "Default");
					if ((name != null) && !name.isEmpty()) {
						CenterController.this.showProgressBar("Insert material");

						try {
							// FIXME IParamMaterial
							parent.addParam(new ParamMaterial(name));
							CenterController.this.dirty = true;
						} catch (Exception e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}

						CenterController.this.refreshViews();
						CenterController.this.hideProgressBar();
					}
				}
			}
		}.start();
	}

	public void mnu_edit_insert_function() {
		new Thread() {
			@Override
			public void run() {
				JLibraryChooser chooser;
				String name;
				ParamFunction func = null;
				ParamContainer parent = null;

				parent = CenterController.this.getParent();
				if (parent != null) {
					name = (String) JOptionPane.showInputDialog(CenterController.this.getGuiComponent(), "Name:",
							"Edit insert function", JOptionPane.QUESTION_MESSAGE, null, null, "Default");
					if ((name != null) && !name.isEmpty()) {
						chooser = new JLibraryChooser();
						if (chooser.showOpenDialog(
								CenterController.this.getGuiComponent()) == JFileChooser.APPROVE_OPTION) {
							CenterController.this.showProgressBar("Insert function");

							try {
								if (chooser.getSelectedType().equals(JLibraryChooser.MATLAB)) {
									func = new ParamMatlabFunction(name, null, chooser.getSelectedFile().getPath(),
											chooser.getSelectedClass());
								} else if (chooser.getSelectedType().equals(JLibraryChooser.PYTHON)) {
									func = new ParamPythonFunction(name, null, chooser.getSelectedFile().getPath(),
											chooser.getSelectedClass());
								} else {
									func = new ParamJavaFunction(name, null, chooser.getSelectedFile().getPath(),
											chooser.getSelectedClass());
								}

								if (func != null) {
									parent.addParam(func);
									CenterController.this.dirty = true;
								}
								CenterController.this.refreshViews();
							} catch (Exception e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							}

							CenterController.this.hideProgressBar();
						}
					}
				}
			}
		}.start();
	}

	public void mnu_edit_insert_template() {
		new Thread() {
			@Override
			public void run() {
				JProjectChooser pc;
				File file = null;
				ParamContainer parent;

				parent = CenterController.this.getParent();
				if (parent != null) {
					pc = new JProjectChooser(JProjectChooser.TEMPLATE_ONLY, new File(Project.getLibraryPath()));
					if (pc.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
						file = pc.getSelectedFile();
					}

					if (file != null) {
						CenterController.this.showProgressBar("Insert template");

						try {
							// TODO IProject
							((Project) CenterController.this.project).importTemplate(new MopFile(file),
									parent.getIdentifier());
						} catch (Exception e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}

						CenterController.this.dirty = true;
						CenterController.this.refreshViews();
						CenterController.this.hideProgressBar();
					}
				}
			}
		}.start();
	}

	private ParamContainer getParent() {
		ParamContainer parent = null;

		if (CenterController.this.selected != null) {
			if (CenterController.this.selected instanceof IParamContainer) {
				// TODO IParamContainer
				parent = (ParamContainer) CenterController.this.selected;
			} else {
				// TODO IParamContainer
				try {
					parent = (ParamContainer) CenterController.this.selected.getParentParameter();
				} catch (RemoteException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}

		// FIXME IParamContainer
		return parent;
	}

	public void mnu_edit_cut_parameter() {
		new Thread() {
			@Override
			public void run() {
				IParameter parent;

				if (CenterController.this.selected != null) {
					CenterController.this.showProgressBar("Cut parameter");

					try {
						parent = CenterController.this.selected.getParentParameter();
						// TODO IParameter
						if (((Parameter) CenterController.this.selected).cut()) {
							CenterController.this.selected = parent;
							CenterController.this.dirty = true;
							CenterController.this.refreshViews();
						}
					} catch (RemoteException e) {
						e.printStackTrace();
					}

					CenterController.this.hideProgressBar();
				}
			}
		}.start();

	}

	public void mnu_edit_copy_parameter() {
		new Thread() {
			@Override
			public void run() {
				if (CenterController.this.selected != null) {
					CenterController.this.showProgressBar("Copy parameter");

					// TODO IParameter
					((Parameter) CenterController.this.selected).copy();

					CenterController.this.hideProgressBar();
				}
			}
		}.start();
	}

	public void mnu_edit_paste_parameter() {
		new Thread() {
			@Override
			public void run() {
				CenterViewManager view;

				if (CenterController.this.selected != null) {
					CenterController.this.showProgressBar("Paste parameter");

					// TODO IParameter
					if (((Parameter) CenterController.this.selected).paste()) {
						CenterController.this.dirty = true;
						CenterController.this.refreshViews();
					} else {
						view = CenterController.this.getViewManager();
						view.refresh(view.EVENT);
					}

					CenterController.this.hideProgressBar();
				}
			}
		}.start();
	}

	public void mnu_edit_delete_parameter() {
		boolean delete = true;

		switch (JOptionPane.showConfirmDialog(this.getGuiComponent(), "Delete selected parameter?", "Delete Parameter",
				JOptionPane.OK_CANCEL_OPTION)) {
		case JOptionPane.OK_OPTION:
			break;
		default:
			delete = false;
		}

		if (delete) {
			new Thread(new Runnable() {
				@Override
				public void run() {
					IParameter sel;

					CenterController.this.showProgressBar("Delete parameter");

					try {
						sel = CenterController.this.selected;
						CenterController.this.selected = CenterController.this.selected.getParentParameter();
						sel.removeParam();
					} catch (RemoteException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}

					CenterController.this.dirty = true;
					CenterController.this.refreshViews();

					CenterController.this.hideProgressBar();
				}
			}).start();
		}
	}

	public void mnu_edit_convert_to_container() {
		new Thread() {
			@Override
			public void run() {
				if (CenterController.this.selected != null) {
					CenterController.this.showProgressBar("Convert to container");

					CenterController.this.selected = ((ParamContainer) CenterController.this.selected)
							.convertToParamContainer();

					CenterController.this.refreshViews();
					CenterController.this.hideProgressBar();
				}
			}
		}.start();
	}

	public void mnu_edit_convert_to_function() {
		new Thread() {
			@Override
			public void run() {
				JLibraryChooser chooser;

				if (CenterController.this.selected != null) {
					chooser = new JLibraryChooser();
					if (chooser
							.showOpenDialog(CenterController.this.getGuiComponent()) == JFileChooser.APPROVE_OPTION) {
						CenterController.this.showProgressBar("Convert to function");

						try {
							if (chooser.getSelectedType().equals(JLibraryChooser.MATLAB)) {
								CenterController.this.selected = ((ParamContainer) CenterController.this.selected)
										.convertToParamMatlabFunction(chooser.getSelectedFile().getPath(),
												chooser.getSelectedClass());
							} else if (chooser.getSelectedType().equals(JLibraryChooser.PYTHON)) {
								CenterController.this.selected = ((ParamContainer) CenterController.this.selected)
										.convertToParamPythonFunction(chooser.getSelectedFile().getPath(),
												chooser.getSelectedClass());
							} else {
								CenterController.this.selected = ((ParamContainer) CenterController.this.selected)
										.convertToParamJavaFunction(chooser.getSelectedFile().getPath(),
												chooser.getSelectedClass());
							}
							CenterController.this.refreshViews();
						} catch (Exception e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
						CenterController.this.hideProgressBar();
					}
				}
			}
		}.start();
	}

	public void mnu_edit_convert_to_unlinked_material() {
		new Thread() {
			@Override
			public void run() {
				if (CenterController.this.selected != null) {
					CenterController.this.showProgressBar("Convert to unlinked material");

					CenterController.this.selected = ((ParamMaterial) CenterController.this.selected)
							.convertToParamContainer().convertToParamMaterial();

					CenterController.this.refreshViews();
					CenterController.this.hideProgressBar();
				}
			}
		}.start();
	}

	public void mnu_edit_convert_to_material() {
		new Thread() {
			@Override
			public void run() {
				if (CenterController.this.selected != null) {

					CenterController.this.showProgressBar("Convert to material");

					CenterController.this.selected = ((ParamContainer) CenterController.this.selected)
							.convertToParamMaterial();

					CenterController.this.refreshViews();
					CenterController.this.hideProgressBar();
				}
			}
		}.start();
	}

	public void mnu_edit_convert_to_collector() {
		new Thread() {
			@Override
			public void run() {
				if (CenterController.this.selected != null) {
					CenterController.this.showProgressBar("Convert to collector");

					CenterController.this.selected = ((ParamContainer) CenterController.this.selected)
							.convertToParamCollector();

					CenterController.this.refreshViews();
					CenterController.this.hideProgressBar();
				}
			}
		}.start();
	}

	public void mnu_edit_convert_to_field() {
		new Thread() {
			@Override
			public void run() {
				if (CenterController.this.selected != null) {
					CenterController.this.showProgressBar("Convert to field");

					CenterController.this.selected = ((ParamField) CenterController.this.selected)
							.convertToParamField();
					CenterController.this.refreshViews();

					CenterController.this.hideProgressBar();
				}
			}
		}.start();
	}

	public void mnu_edit_convert_to_formula() {
		new Thread() {
			@Override
			public void run() {
				if (CenterController.this.selected != null) {
					CenterController.this.showProgressBar("Convert to formula");

					CenterController.this.selected = ((ParamField) CenterController.this.selected)
							.convertToParamFormula();

					CenterController.this.refreshViews();
					CenterController.this.hideProgressBar();
				}
			}
		}.start();
	}

	public void mnu_edit_convert_to_combobox() {
		new Thread() {
			@Override
			public void run() {
				if (CenterController.this.selected != null) {
					CenterController.this.showProgressBar("Convert to combobox");

					CenterController.this.selected = ((ParamField) CenterController.this.selected)
							.convertToParamComboBox();

					CenterController.this.refreshViews();
					CenterController.this.hideProgressBar();
				}
			}
		}.start();
	}

	public void mnu_edit_convert_to_linked_field() {
		new Thread() {
			@Override
			public void run() {
				if (CenterController.this.selected != null) {
					CenterController.this.showProgressBar("Convert to field");

					CenterController.this.selected = ((ParamField) CenterController.this.selected)
							.convertToParamLinkedField();

					CenterController.this.refreshViews();
					CenterController.this.hideProgressBar();
				}
			}
		}.start();
	}

	public void mnu_edit_duplicate() {
		int count = 0;

		try {
			count = Integer.parseInt((String) JOptionPane.showInputDialog(this.getGuiComponent(),
					"Number of indices to create", "Duplicate index", JOptionPane.QUESTION_MESSAGE, null, null, 1));
		} catch (Exception e) {
		}

		if (count > 0) {
			this.mnu_edit_duplicate(count);
		}
	}

	public void mnu_edit_duplicate(final int count) {
		new Thread() {
			@Override
			public void run() {
				CenterController.this.showProgressBar("Dulplicate", count - 1);

				if (CenterController.this.project != null) {
					try {
						for (int k = 0; k < count; k++) {
							CenterController.this.updateProgressBar(k, "Duplicate");
							CenterController.this.project.duplicate();
							CenterController.this.dirty = true;
							SwingUtilities.invokeLater(new Runnable() {
								@Override
								public void run() {
									CenterViewManager view;
									view = CenterController.this.getViewManager();
									view.refresh(view.INDICES);
								}
							});
						}
					} catch (RemoteException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					} catch (CloneException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					} catch (IOException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
				CenterController.this.refreshViews();
				CenterController.this.hideProgressBar();
			}

		}.start();
	}

	public void mnu_edit_reduce() {
		int count = 0;

		try {
			count = Integer.parseInt((String) JOptionPane.showInputDialog(this.getGuiComponent(),
					"Number of indices to remove", "Reduce index", JOptionPane.QUESTION_MESSAGE, null, null, 1));
		} catch (Exception e) {
		}

		if (count > 0) {
			this.mnu_edit_reduce(count);
		}
	}

	public void mnu_edit_reduce(final int count) {
		new Thread() {
			@Override
			public void run() {
				CenterController.this.showProgressBar("Reduce to formula", count - 1);

				if (CenterController.this.project != null) {
					try {
						for (int k = 0; k < count; k++) {
							CenterController.this.updateProgressBar(k, "Reduce");
							CenterController.this.project.reduce();
							CenterController.this.dirty = true;
							SwingUtilities.invokeLater(new Runnable() {
								@Override
								public void run() {
									CenterViewManager view;
									view = CenterController.this.getViewManager();
									view.refresh(view.INDICES);
								}
							});
						}
					} catch (RemoteException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}

				CenterController.this.refreshViews();
				CenterController.this.hideProgressBar();
			}

		}.start();
	}

	public void mnu_update_refresh() {
		new Thread() {
			@Override
			public void run() {
				CenterController.this.showProgressBar("Refresh project");

				if (CenterController.this.project != null) {
					try {
						CenterController.this.project.refresh();
					} catch (RemoteException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}

				CenterController.this.refreshViews();
				CenterController.this.hideProgressBar();
			}
		}.start();
	}

	public void mnu_update_check() {
		new Thread() {
			@Override
			public void run() {
				CenterController.this.showProgressBar("Check project");

				if (CenterController.this.project != null) {
					try {
						CenterController.this.project.check();
					} catch (RemoteException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}

				CenterController.this.refreshViews();
				CenterController.this.hideProgressBar();
			}
		}.start();
	}

	public void mnu_update_regenerate() {
		new Thread() {
			@Override
			public void run() {
				CenterController.this.showProgressBar("Regenerate project");

				if (CenterController.this.project != null) {
					try {
						Map<String, Object> args = new Hashtable<String, Object>();
						args.put("RunSim", CenterController.this.isStartSimulation());
						args.put("RemoteCompute", CenterController.this.isRemoteCompute());
						args.put("Optimize", CenterController.this.isOptimize());
						CenterController.this.project.regenerate(args);
						CenterController.this.dirty = true;
					} catch (RemoteException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}

				CenterController.this.refreshViews();
				CenterController.this.hideProgressBar();
			}
		}.start();
	}

	public void mnu_update_regenerate_all() {
		new Thread() {
			@Override
			public void run() {
				SimulateAll sim;

				CenterController.this.getGuiComponent().setEnabled(false);
				// MagOptController.this.showProgressBar("Update",
				// "Regenerate project");

				if (CenterController.this.project != null) {
					// TODO IProject
					sim = new SimulateAll((Project) CenterController.this.project);
					sim.setRunSimulation(CenterController.this.isStartSimulation());
					sim.setRemoteCompute(CenterController.this.isRemoteCompute());
					sim.setOptimize(CenterController.this.isOptimize());
					// TODO timeout
					// sim.setTimeout(timeout);
					// TODO DISABLE
					// FIXME
					// FIXME JFrame
					new SimulateAllGui(sim, (JFrame) CenterController.this.getGuiComponent());
					CenterController.this.dirty = true;
				}
				CenterController.this.refreshViews();

				CenterController.this.getGuiComponent().setEnabled(true);
				// hideProgressBar();
			}
		}.start();
	}

	public void mnu_update_jobmanager() {
		new Thread() {
			@Override
			public void run() {
				CenterController.this.showProgressBar("Open JobManager");

				JobDesc.setVisible(true);

				CenterController.this.hideProgressBar();
			}
		}.start();
	}

	public void mnu_update_macro() {
		CenterViewManager view;
		Component comp = null;

		view = CenterController.this.getViewManager();
		if (view != null) {
			comp = CenterController.this.getGuiComponent();
		}

		this.mnu_update_macro(new MacroDialog(comp).getMacro());
	}

	public void mnu_update_macro(final File script) {
		new Thread() {
			@Override
			public void run() {
				CenterViewManager view;
				PythonConsole py;
				JepProxy jep;

				CenterController.this.showProgressBar("Run macro");

				view = CenterController.this.getViewManager();
				if (view != null) {
					py = view.pythonConsole;
					if (py != null) {
						if (script != null) {
							try {
								jep = py.getJep();
								if (CenterController.this.project != null) {
									jep.eval("from symspace.wrapper.symwrapper import SymParameter");
									jep.set("__project__", CenterController.this.project);
									jep.eval("__project__ = SymParameter._convertObject(__project__)");
									jep.set("__parameter__", CenterController.this.selected);
									jep.eval("__parameter__ = SymParameter._convertObject(__parameter__)");
									jep.eval("del SymParameter");
								}
								py.getConsole().writeInput(
										"Run macro: " + script.getName() + " (" + script.getParent() + ")\n");
								jep.eval("exec(open(r'" + script.getAbsolutePath() + "').read())");
								if (CenterController.this.project != null) {
									jep.eval("del __project__");
									jep.eval("del __parameter__");
								}
							} catch (Exception ex) {
								py.getConsole().getErrorPrintStream().println("Error by run macro (" + script + ")");
								py.getConsole().getErrorPrintStream().println(ex.getMessage());
								ex.printStackTrace(py.getConsole().getErrorPrintStream());
							}
						}
					} else {
						System.err.println("Python console not available");
					}
				}

				CenterController.this.hideProgressBar();
			}
		}.start();

	}

	public void mnu_simulation_start() {
		CenterController.this.startSimulation = !CenterController.this.startSimulation;

		this.refreshViews();
	}

	public void mnu_simulation_remote() {
		CenterController.this.remoteCompute = !CenterController.this.remoteCompute;

		this.refreshViews();
	}

	public void mnu_simulation_optimize() {
		CenterController.this.optimze = !CenterController.this.optimze;

		this.refreshViews();
	}

	public void mnu_documentation_create_pdf_report() {

		new Thread() {
			@Override
			public void run() {
				ReportStructureDialog dialog;
				ReportStructure structure;

				dialog = new ReportStructureDialog();
				structure = dialog.show(null);

				if (structure != null) {
					CenterController.this.showProgressBar("Create pdf report");

					try {
						// TODO IProject
						((Project) CenterController.this.project).createTexReport(structure);
						CenterController.this.dirty = true;

					} catch (IOException ex) {
						try {
							CenterController.this.project.error(ex.getMessage(), ex.getStackTrace());
						} catch (RemoteException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
					}
					CenterController.this.hideProgressBar();
				}
			}
		}.start();
	}

	public void mnu_documentation_fill_pdf_report() {
		new Thread() {
			@Override
			public void run() {
				boolean create = true;
				final boolean save;
				JFileChooser chooser;

				// FIXME save??
				switch (JOptionPane.showConfirmDialog(CenterController.this.getGuiComponent(),
						"Save Project to continue?", "Create documentation", JOptionPane.OK_CANCEL_OPTION)) {
				case JOptionPane.OK_OPTION:
					save = true;
					break;
				default:
					create = false;
					save = false;
				}

				if (create) {
					chooser = new JFileChooser();
					chooser.setFileFilter(new FileNameExtensionFilter("Report Files (*.tex)", "tex"));
					chooser.setDialogTitle("Select Report-file");
					try {
						chooser.setCurrentDirectory(CenterController.this.project.getMopFile().getParentFile());
					} catch (RemoteException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					if ((chooser.showOpenDialog(null) == JFileChooser.APPROVE_OPTION)
							&& chooser.getSelectedFile().exists()) {
						CenterController.this.showProgressBar("Fill pdf report");

						if (save) {
							try {
								CenterController.this.project.save();
								// MagOptController.this.dirty = false;
							} catch (Exception e) {
								// FIXME
								System.err.println(e.getMessage());
								e.printStackTrace();
							}

							// MagOptController.this.refreshViews();
						}

						// TODO IProject
						new FillTexReport(chooser.getSelectedFile(), (Project) CenterController.this.project).show();

						CenterController.this.dirty = true;
						CenterController.this.refreshViews();
						CenterController.this.hideProgressBar();
					}
				}
			}
		}.start();
	}

	public void mnu_documentation_create_excel_report() {
		new Thread() {
			@Override
			public void run() {
				JFileChooser chooser;
				File file;

				chooser = new JFileChooser();
				chooser.setFileFilter(new FileNameExtensionFilter("Excel files (*.xls, *.xlsx)", "xls", "xlsx"));
				chooser.setDialogTitle("Select Location for Excel File");
				try {
					chooser.setCurrentDirectory(CenterController.this.project.getMopFile().getParentFile());
				} catch (RemoteException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				if ((chooser.showOpenDialog(null) == JFileChooser.APPROVE_OPTION)
						&& !chooser.getSelectedFile().getPath().isEmpty()) {
					CenterController.this.showProgressBar("Create excel report");

					try {
						// TODO IProject
						file = new ReportExcel().reportExcel((Project) CenterController.this.project,
								chooser.getSelectedFile());
						CenterController.this.dirty = true;

						if (Desktop.isDesktopSupported()) {
							Desktop.getDesktop().open(file);
						}

					} catch (IOException ex) {
						try {
							CenterController.this.project.error(ex.getMessage(), ex.getStackTrace());
						} catch (RemoteException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
					}

					CenterController.this.hideProgressBar();
				}
			}
		}.start();
	}

	public void mnu_documentation_fill_excel_report() {
		new Thread() {
			@Override
			public void run() {
				JFileChooser chooserOpen, chooserExport;
				FileNameExtensionFilter filterExport;
				File file;

				chooserOpen = new JFileChooser();
				chooserOpen.setFileFilter(new FileNameExtensionFilter("Excel files (*.xls, *.xlsx)", "xls", "xlsx"));
				chooserOpen.setDialogTitle("Select Location for Excel Template");
				try {
					chooserOpen.setCurrentDirectory(CenterController.this.project.getMopFile().getParentFile());
				} catch (RemoteException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				if ((chooserOpen.showOpenDialog(null) == JFileChooser.APPROVE_OPTION)
						&& chooserOpen.getSelectedFile().exists()) {
					if (chooserOpen.getSelectedFile().getName().toLowerCase().endsWith(".xlsx")) {
						filterExport = new FileNameExtensionFilter("Excel files (*.xlsx)", "xlsx");
					} else {
						filterExport = new FileNameExtensionFilter("Excel files (*.xls)", "xls");
					}
					chooserExport = new JFileChooser();
					chooserExport.setFileFilter(filterExport);
					chooserExport.setDialogTitle("Select Location for Excel File");
					chooserExport.setCurrentDirectory(chooserOpen.getCurrentDirectory());
					if ((chooserExport.showOpenDialog(null) == JFileChooser.APPROVE_OPTION)) {

						// FIXME Excel Matlab

						CenterController.this.showProgressBar("Fill excel report");

						try {
							// TODO IProject
							file = new FillExcelReport().fillExcel((Project) CenterController.this.project,
									chooserOpen.getSelectedFile(), chooserExport.getSelectedFile());
							CenterController.this.dirty = true;

							if (Desktop.isDesktopSupported()) {
								Desktop.getDesktop().open(file);
							}
						} catch (IOException ex) {
							try {
								CenterController.this.project.error(ex.getMessage(), ex.getStackTrace());
							} catch (RemoteException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							}
						}

						CenterController.this.hideProgressBar();
					}
				}
			}
		}.start();
	}

	public void mnu_documentation_docfile_generate_all() {
		new Thread() {
			@Override
			public void run() {
				CenterController.this.showProgressBar("Generate all docfiles");

				// TODO IProject
				((Project) CenterController.this.project).generateDocumentationFiles();
				CenterController.this.dirty = true;

				CenterController.this.refreshViews();
				CenterController.this.hideProgressBar();
			}
		}.start();
	}

	public void mnu_documentation_docfile_generate_selected() {
		new Thread() {
			@Override
			public void run() {
				CenterController.this.showProgressBar("Generate selected docfiles");

				// TODO IProject, IParameter
				((Project) CenterController.this.project)
						.generateDocumentationFiles((Parameter) CenterController.this.selected);
				CenterController.this.dirty = true;

				CenterController.this.refreshViews();
				CenterController.this.hideProgressBar();
			}
		}.start();
	}

	public void mnu_documentation_docfile_reset_all() {
		new Thread() {
			@Override
			public void run() {
				CenterController.this.showProgressBar("Reset all docfiles");

				// TODO IProject
				((Project) CenterController.this.project).resetDocumentationFiles();
				CenterController.this.dirty = true;

				CenterController.this.refreshViews();
				CenterController.this.hideProgressBar();
			}
		}.start();
	}

	public void mnu_documentation_docfile_reset_selected() {
		new Thread() {
			@Override
			public void run() {
				CenterController.this.showProgressBar("Reset selected docfiles");

				// TODO IProject, IParameter
				((Project) CenterController.this.project)
						.resetDocumentationFiles((Parameter) CenterController.this.selected);
				CenterController.this.dirty = true;

				CenterController.this.refreshViews();
				CenterController.this.hideProgressBar();
			}
		}.start();
	}

	public void mnu_documentation_docfile_delete_all() {
		new Thread() {
			@Override
			public void run() {
				CenterController.this.showProgressBar("Delete all docfiles");

				// TODO IProject
				((Project) CenterController.this.project).deleteDocumentationFiles();
				CenterController.this.dirty = true;

				CenterController.this.refreshViews();
				CenterController.this.hideProgressBar();
			}
		}.start();
	}

	public void mnu_documentation_docfile_delete_selected() {
		new Thread() {
			@Override
			public void run() {
				CenterController.this.showProgressBar("Delete selected docfiles");

				// TODO IProject, IParameter
				((Project) CenterController.this.project)
						.deleteDocumentationFiles((Parameter) CenterController.this.selected);
				CenterController.this.dirty = true;

				CenterController.this.refreshViews();
				CenterController.this.hideProgressBar();
			}
		}.start();
	}

	public boolean isOfflineMaterial() {
		// TODO IParamMaterial
		return ParamMaterial.isOffline();
	}

	public void mnu_extra_material_change_user() {
		new Thread() {
			@Override
			public void run() {
				SelectUserDialog dia = new SelectUserDialog(null, ParamMaterial.getDbUser());
				dia.show();

				// TODO IParamMaterial
				ParamMaterial.setDbUser(dia.getUser(), dia.getPassword());
			}
		}.start();

	}

	public void mnu_extra_material_reset_user() {
		// TODO IParamMaterial
		ParamMaterial.resetDbUser();
	}

	public void mnu_extra_material_use_file() {
		new Thread() {
			@Override
			public void run() {
				JFileChooser choose;

				if (CenterController.this.isOfflineMaterial()) {
					// TODO IParamMaterial
					ParamMaterial.setDbPath(null);
				} else {
					choose = new JFileChooser(MagOptConfig.getHome());
					choose.setFileFilter(new FileNameExtensionFilter("Material File (*.mat)", "mat"));
					choose.setDialogTitle("Select Material File");
					choose.setFileSelectionMode(JFileChooser.FILES_ONLY);
					choose.setMultiSelectionEnabled(false);
					if (choose.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
						// TODO IParamMaterial
						ParamMaterial.setDbPath(choose.getSelectedFile().getPath());
					}
				}

				CenterController.this.refreshViews();
			}
		}.start();

	}

	public void mnu_extra_material_create_file() {
		// TODO IParamMaterial
		new Thread() {
			@Override
			public void run() {
				JFileChooser choose;
				File file;

				choose = new JFileChooser(MagOptConfig.getHome());
				choose.setFileFilter(new FileNameExtensionFilter("Material File (*.mat)", "mat"));
				choose.setDialogTitle("Select Material File");
				choose.setFileSelectionMode(JFileChooser.FILES_ONLY);
				choose.setMultiSelectionEnabled(false);
				if (choose.showSaveDialog(null) == JFileChooser.APPROVE_OPTION) {
					file = choose.getSelectedFile();
					if (!file.getName().toLowerCase().endsWith(".mat")) {
						file = new File(file.getAbsolutePath() + ".mat");
					}

					CenterController.this.showProgressBar("Create material-file");

					try {
						// TODO IParamMaterial
						FileConnection.createFile(file, ParamMaterial.getConnection());
						CenterController.this.hideProgressBar();
					} catch (Exception e) {
						CenterController.this.hideProgressBar();
						JOptionPane.showMessageDialog(null, e.getMessage(), "Error by creating material file",
								JOptionPane.ERROR_MESSAGE);
					}
				}
			}
		}.start();
	}

	public void mnu_extra_variants() {
		// boolean open = true;
		IProject p;

		p = new VariantsDialog(this.getGuiComponent(), this.project).getResultProject();
		if (p != null) {
			if (p != this.project) {
				try {
					p.save();
					// } catch (Exception e) {
					// System.err.println(e.getMessage());
					// e.printStackTrace();
					// }
					// switch (JOptionPane.showConfirmDialog(this.gui,
					// "Save loaded project?", "Open varians-project",
					// JOptionPane.YES_NO_CANCEL_OPTION)) {
					// case JOptionPane.YES_OPTION:
					// this.mnu_file_save();
					// break;
					// case JOptionPane.NO_OPTION:
					// break;
					// default:
					// open = false;
					// }

					// if (open) {
					// try {
					CenterController.openProjectExtern(p.getMopFile());

					// MagOptController.this.project = p;
					// MagOptController.this.selected =
					// MagOptController.this.project;
					// MagOptController.this.dirty = false;
					// } catch (Exception e) {
					// System.err.println(e.getMessage());
					// e.printStackTrace();
					// }
					//
					// try {
					// new Storage().addRecentProject(
					// p.getMopFile().getAbsolutePath());
				} catch (Exception e1) {
					System.err.println(e1.getMessage());
					e1.printStackTrace();
				}

				// }
			}
			this.refreshViews();
		}
	}

	public void mnu_extra_analysis() {
		new AnalysisDialog(this.getGuiComponent(), this.project);
	}

	public void mnu_debug_python_assign_project() {
		new Thread() {
			@Override
			public void run() {
				SendToDialog dia;
				JepProxy jep;
				CenterViewManager view;

				try {
					dia = new SendToDialog((JFrame) CenterController.this.getGuiComponent());
					dia.show("Assign project handle to Python console", "CurProject");
					if (dia.getVariableName() != null) {

						CenterController.this.showProgressBar("Assign project to python");

						view = CenterController.this.getViewManager();
						jep = view.pythonConsole.getJep();
						if (jep == null) {
							CenterController.this.project.warning("No python-console avaiable");
						} else {
							jep.eval("from symspace.wrapper.symwrapper import SymParameter");
							jep.set(dia.getVariableName(), CenterController.this.project);
							jep.eval(dia.getVariableName() + " = SymParameter._convertObject(" + dia.getVariableName()
									+ ")");
							jep.eval("del SymParameter");
						}
					}
				} catch (Exception ex) {
					try {
						CenterController.this.project.warning(ex.getMessage(), ex.getStackTrace());
					} catch (RemoteException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				} finally {
					CenterController.this.hideProgressBar();
				}
			}
		}.start();
	}

	public void mnu_debug_python_assign_param() {
		new Thread() {
			@Override
			public void run() {
				SendToDialog dia;
				JepProxy jep;
				CenterViewManager view;

				try {
					dia = new SendToDialog((JFrame) CenterController.this.getGuiComponent());
					dia.show("Assign object handle to Python console", "CurObject");
					if (dia.getVariableName() != null) {
						CenterController.this.showProgressBar("Assign parameter to python");

						view = CenterController.this.getViewManager();
						jep = view.pythonConsole.getJep();
						if (jep == null) {
							CenterController.this.selected.warning("No python-console avaiable");
						} else {
							jep.eval("from symspace.wrapper.symwrapper import SymParameter");
							jep.set(dia.getVariableName(), CenterController.this.selected);
							jep.eval(dia.getVariableName() + " = SymParameter._convertObject(" + dia.getVariableName()
									+ ")");
							jep.eval("del SymParameter");
						}
					}
				} catch (Exception ex) {
					try {
						CenterController.this.selected.warning(ex.getMessage(), ex.getStackTrace());
					} catch (RemoteException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				} finally {
					CenterController.this.hideProgressBar();
				}
			}
		}.start();
	}

	public void mnu_debug_python_assign_data() {
		new Thread() {
			@Override
			public void run() {
				SendToDialog dia;
				JepProxy jep;
				CenterViewManager view;

				try {
					dia = new SendToDialog((JFrame) CenterController.this.getGuiComponent());
					dia.show("Assign data to Python console", CenterController.this.selected.getName());
					if (dia.getVariableName() != null) {
						CenterController.this.showProgressBar("Assign data to python");

						view = CenterController.this.getViewManager();
						jep = view.pythonConsole.getJep();
						if (jep == null) {
							CenterController.this.selected.warning("No python-console avaiable");
						} else {
							jep.eval("from lcm.tdata_interface.pyjdata_interface import PyJDataInterface");
							jep.set(dia.getVariableName(), CenterController.this.selected.getData());
							jep.eval(dia.getVariableName() + " = PyJDataInterface.convert_J2Py(" + dia.getVariableName()
									+ ")");
							jep.eval("del PyJDataInterface");
						}
					}
				} catch (Exception ex) {
					try {
						CenterController.this.selected.warning(ex.getMessage(), ex.getStackTrace());
					} catch (RemoteException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				} finally {
					CenterController.this.hideProgressBar();
				}
			}
		}.start();

	}

	public void mnu_debug_python_get_data() {
		new Thread() {
			@Override
			public void run() {
				GetFromDialog dia;
				JepProxy jep;
				CenterViewManager view;

				try {
					dia = new GetFromDialog(null);

					view = CenterController.this.getViewManager();
					jep = view.pythonConsole.getJep();
					if (jep == null) {
						CenterController.this.selected.warning("No python-console avaiable");
					} else {

						jep.eval("_variables_ = getVariables(vars())");
						@SuppressWarnings("unchecked")
						List<String> vars = (List<String>) jep.getValue("_variables_");
						jep.eval("del _variables_");
						dia.show("Get data from Python console", vars.toArray(new String[0]),
								CenterController.this.selected.getName());
						if (dia.getVariableName() != null) {
							CenterController.this.showProgressBar("Get data from python");

							jep.eval("from lcm.tdata_interface.pyjdata_interface import PyJDataInterface");
							jep.eval(dia.getVariableName() + "_raw = PyJDataInterface.convert_Py2J("
									+ dia.getVariableName() + ")");
							CenterController.this.selected
									.setData((TData) jep.getValue(dia.getVariableName() + "_raw"));
							jep.eval("del " + dia.getVariableName() + "_raw");
							jep.eval("del PyJDataInterface");

							CenterController.this.refreshViews();
						}
					}
				} catch (Exception ex) {
					try {
						CenterController.this.selected.warning(ex.getMessage(), ex.getStackTrace());
					} catch (RemoteException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				} finally {
					CenterController.this.hideProgressBar();
				}
			}
		}.start();

	}

	public void mnu_debug_python_write_data() {
		new Thread() {
			@Override
			public void run() {
				JFileChooser dia;
				JepProxy jep;
				CenterViewManager view;

				try {
					dia = new JFileChooser();

					FileNameExtensionFilter fpickle = new FileNameExtensionFilter("Pickle-file (*.p)", "p");

					dia.addChoosableFileFilter(fpickle);
					dia.addChoosableFileFilter(new FileNameExtensionFilter("All-file (*.*)", "*"));
					dia.setFileFilter(fpickle);

					dia.setAcceptAllFileFilterUsed(false);
					dia.setDialogTitle("Choose Pickle File");
					dia.setFileSelectionMode(JFileChooser.FILES_ONLY);
					dia.setMultiSelectionEnabled(false);
					dia.setFileHidingEnabled(true);

					if (dia.showSaveDialog(CenterController.this.getGuiComponent()) == JFileChooser.APPROVE_OPTION) {
						CenterController.this.showProgressBar("Assign data to python");

						view = CenterController.this.getViewManager();
						jep = view.pythonConsole.getJep();
						if (jep == null) {
							CenterController.this.selected.warning("No python-console avaiable");
						} else {
							jep.eval("from lcm.tdata_interface.pyjdata_interface import PyJDataInterface");
							jep.eval("import pickle");
							jep.set("pickle_data", CenterController.this.selected.getData());
							jep.eval("pickle_data = PyJDataInterface.convert_J2Py(pickle_data)");
							jep.eval("pickle_fh = open('" + dia.getSelectedFile().getPath().replace("\\", "\\\\")
									+ "','wb')");
							jep.eval("pickle.dump(pickle_data,pickle_fh)");
							jep.eval("pickle_fh.close()");
							jep.eval("del pickle_data");
							jep.eval("del pickle_fh");
							jep.eval("del PyJDataInterface");
							jep.eval("del pickle");
						}
					}
				} catch (Exception ex) {
					try {
						CenterController.this.selected.warning(ex.getMessage(), ex.getStackTrace());
					} catch (RemoteException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				} finally {
					CenterController.this.hideProgressBar();
				}
			}
		}.start();

	}

	public void mnu_debug_python_restart() {
		new Thread() {
			@Override
			public void run() {
				CenterController.this.showProgressBar("Restart python connection");

				try {
					CenterController.this.project.restartJep();
				} catch (Exception ex) {
					System.err.println(ex.getMessage());
					ex.printStackTrace();
				}

				CenterController.this.hideProgressBar();
			}
		}.start();
	}

	public void mnu_debug_python_assign_system_jep() {
		new Thread() {
			@Override
			public void run() {
				JepProxy jep;
				CenterViewManager view;

				CenterController.this.showProgressBar("Assign data to python");

				try {
					jep = ((Project) CenterController.this.project).getJep();

					view = CenterController.this.getViewManager();
					view.pythonConsole.restart(jep);
				} catch (Exception ex) {
					try {
						CenterController.this.project.warning(ex.getMessage(), ex.getStackTrace());
					} catch (RemoteException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}

				CenterController.this.hideProgressBar();
			}
		}.start();

	}

	public void mnu_debug_matlab_assign_project() {
		new Thread() {
			@Override
			public void run() {
				SendToDialog dia;
				MatlabProxy proxy;

				try {
					dia = new SendToDialog((JFrame) CenterController.this.getGuiComponent());
					dia.show("Assign project handle to Matlab workspace", "CurProject");
					if (dia.getVariableName() != null) {
						CenterController.this.showProgressBar("Assign project to matlab");

						proxy = MatlabControl.getProxy();
						// TODO IProject
						proxy.setVariable(dia.getVariableName(),
								((Project) CenterController.this.project).getRMIImpl().getClientImpl());
						proxy.eval(dia.getVariableName() + " = Parameter.getObject(" + dia.getVariableName() + ");");
					}
				} catch (Exception ex) {
					try {
						CenterController.this.project.warning(ex.getMessage(), ex.getStackTrace());
					} catch (RemoteException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				} finally {
					CenterController.this.hideProgressBar();
				}
			}
		}.start();
	}

	public void mnu_debug_matlab_assign_param() {
		new Thread() {
			@Override
			public void run() {
				SendToDialog dia;
				MatlabProxy proxy;

				try {
					dia = new SendToDialog((JFrame) CenterController.this.getGuiComponent());
					dia.show("Assign object handle to Matlab workspace", "CurObject");
					if (dia.getVariableName() != null) {
						CenterController.this.showProgressBar("Assign parameter to matlab");

						proxy = MatlabControl.getProxy();
						// TODO IParameter
						proxy.setVariable(dia.getVariableName(),
								((Parameter) CenterController.this.selected).getRMIImpl().getClientImpl());
						proxy.eval(dia.getVariableName() + " = Parameter.getObject(" + dia.getVariableName() + ");");
					}
				} catch (Exception ex) {
					try {
						CenterController.this.selected.warning(ex.getMessage(), ex.getStackTrace());
					} catch (RemoteException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				} finally {
					CenterController.this.hideProgressBar();
				}
			}
		}.start();

	}

	public void mnu_debug_matlab_assign_data() {
		new Thread() {
			@Override
			public void run() {
				SendToDialog dia;
				MatlabProxy proxy;

				try {
					dia = new SendToDialog((JFrame) CenterController.this.getGuiComponent());
					dia.show("Assign object handle to Matlab workspace", CenterController.this.selected.getName());
					if (dia.getVariableName() != null) {
						CenterController.this.showProgressBar("Assign data to matlab");

						proxy = MatlabControl.getProxy();
						proxy.setVariable(dia.getVariableName(), CenterController.this.selected.getData());
						proxy.eval(dia.getVariableName() + " = dtypes.Generic.getMatlabVariableFromGeneric("
								+ dia.getVariableName() + ");");
					}
				} catch (Exception ex) {
					try {
						CenterController.this.selected.warning(ex.getMessage(), ex.getStackTrace());
					} catch (RemoteException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}

				} finally {
					CenterController.this.hideProgressBar();
				}
			}
		}.start();
	}

	public void mnu_debug_matlab_get_data() {
		new Thread() {
			@Override
			public void run() {
				GetFromDialog dia;
				MatlabProxy proxy;

				try {
					dia = new GetFromDialog(null);
					proxy = MatlabControl.getProxy();
					String[] vars = (String[]) proxy.returningEval("who", 1)[0];
					dia.show("Get data from Matlab workspace", vars, CenterController.this.selected.getName());
					if (dia.getVariableName() != null) {
						CenterController.this.showProgressBar("Get data from matlab");

						CenterController.this.selected.setData((TData) proxy.returningEval(
								"dtypes.Generic.getGenericFromMatlabVariable(" + dia.getVariableName() + ");", 1)[0]);

						CenterController.this.refreshViews();
					}
				} catch (Exception ex) {
					try {
						CenterController.this.selected.warning(ex.getMessage(), ex.getStackTrace());
					} catch (RemoteException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				} finally {
					CenterController.this.hideProgressBar();
				}
			}
		}.start();
	}

	public void mnu_debug_extract_index() {
		// boolean open = true;
		String name, ext = "";
		int index;

		// if (this.project != null) {
		// switch (JOptionPane.showConfirmDialog(this.gui,
		// "Save current project?", "Extract index",
		// JOptionPane.YES_NO_CANCEL_OPTION)) {
		// case JOptionPane.YES_OPTION:
		// this.mnu_file_save();
		// break;
		// case JOptionPane.NO_OPTION:
		// break;
		// default:
		// open = false;
		// }
		// }

		// if (open) {
		JProjectChooser chooser = new JProjectChooser(JProjectChooser.ALL, new File("."));
		try {
			// generate new name
			name = this.project.getMopFile().getName();
			index = name.lastIndexOf('.');
			if (index != -1) {
				ext = name.substring(index);
				name = name.substring(0, index);
			}
			name += "_Index" + (this.project.getIndex() + 1) + ext;

			chooser.setSelectedFile(new File(this.project.getMopFile().getParentFile(), name));
			if (chooser.showOpenDialog(this.getGuiComponent()) == JFileChooser.APPROVE_OPTION) {
				this.mnu_debug_extract_index(chooser.getSelectedFile());
			}
		} catch (RemoteException e) {
			System.err.println(e.getMessage());
			e.printStackTrace();
		}
		// }

	}

	public void mnu_debug_extract_index(final File file) {
		new Thread() {
			@Override
			public void run() {
				IProject proj;

				CenterController.this.showProgressBar("Extract current index");

				try {
					proj = CenterController.this.project.extractIndex(CenterController.this.project.getIndex(), file);
					proj.save();
					CenterController.openProjectExtern(proj.getMopFile());

					// MagOptController.this.project =
					// MagOptController.this.project
					// .extractIndex(
					// MagOptController.this.project.getIndex(),
					// file);
					// MagOptController.this.selected =
					// MagOptController.this.project;
					// MagOptController.this.dirty = false;

					// } catch (Exception e) {
					// System.err.println(e.getMessage());
					// e.printStackTrace();
					// }

					// try {
					// new Storage().addRecentProject(file.getAbsolutePath());
				} catch (Exception e1) {
					System.err.println(e1.getMessage());
					e1.printStackTrace();
				}

				CenterController.this.refreshViews();
				CenterController.this.hideProgressBar();
			}
		}.start();

	}

	public void mnu_debug_reset_all() {
		new Thread() {
			@Override
			public void run() {
				CenterController.this.showProgressBar("Reset all functions");

				try {
					CenterController.this.project.resetAllFunctions();
					// TODO
					CenterController.this.project.info("Reset all functions");
				} catch (RemoteException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}

				CenterController.this.refreshViews();
				CenterController.this.hideProgressBar();
			}
		}.start();
	}

	public void mnu_debug_reset_selected() {
		new Thread() {
			@Override
			public void run() {
				CenterController.this.showProgressBar("Reset selected function");

				try {
					if (CenterController.this.selected instanceof IFunction) {
						try {
							((IFunction) CenterController.this.selected).reset();
							CenterController.this.selected.info("Reset " + CenterController.this.selected.getName());
							CenterController.this.refreshViews();
						} catch (RemoteException ex) {
							CenterController.this.selected.warning(ex.getMessage(), ex.getStackTrace());
						}
					}
				} catch (RemoteException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}

				CenterController.this.hideProgressBar();
			}
		}.start();
	}

	public void mnu_debug_update_selected() {
		new Thread() {
			@Override
			public void run() {
				CenterController.this.showProgressBar("Update selected function");

				try {
					if (CenterController.this.selected instanceof IFunction) {
						CenterController.this.selected.info("Update " + CenterController.this.selected.getName());

						Map<String, Object> args = new Hashtable<String, Object>();
						args.put("RunSim", CenterController.this.isStartSimulation());
						args.put("RemoteCompute", CenterController.this.isRemoteCompute());
						args.put("Optimize", CenterController.this.isOptimize());
						try {
							((IFunction) CenterController.this.selected).update(args);
							CenterController.this.refreshViews();
						} catch (Exception ex) {
							CenterController.this.selected.error(ex.getMessage(), ex.getStackTrace());
						}
					}
				} catch (RemoteException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}

				CenterController.this.hideProgressBar();
			}
		}.start();
	}

	public void mnu_debug_compare_project() {
		new Thread() {
			@Override
			public void run() {
				// TODO IProject
				new CompareProject((Project) CenterController.this.project);
			}
		}.start();
	}

	public boolean exist_help() {
		return Project.getHelpFile().exists();
	}

	public void mnu_help_help() {
		new Thread() {
			@Override
			public void run() {
				CenterController.this.showProgressBar("Open help");

				Project.openHelp();

				CenterController.this.hideProgressBar();
				CenterController.this.refreshViews();
			}
		}.start();
	}

	public void mnu_help_help_update() {
		new Thread() {
			@Override
			public void run() {
				CenterController.this.showProgressBar("Update help");

				Project.updateHelp();

				CenterController.this.hideProgressBar();
				CenterController.this.refreshViews();
			}
		}.start();
	}

	public boolean exist_docu() {
		return Project.getDocuFile().exists();
	}

	public void mnu_help_docu() {
		new Thread() {
			@Override
			public void run() {
				CenterController.this.showProgressBar("Open documentation");

				Project.openDocu();

				CenterController.this.hideProgressBar();
				CenterController.this.refreshViews();
			}
		}.start();
	}

	public void mnu_help_docu_update() {
		new Thread() {
			@Override
			public void run() {
				CenterController.this.showProgressBar("Update documentation");

				Project.updateDocu();

				CenterController.this.hideProgressBar();
				CenterController.this.refreshViews();
			}
		}.start();
	}

	public void mnu_help_about() {
		new Thread() {
			@Override
			public void run() {
				// CenterController.this.showProgressBar("Open about");

				new About(CenterController.this.getGuiComponent(), SyMIcons.getIcons()).setVisible(true);

				// CenterController.this.hideProgressBar();
			}
		}.start();
	}

	public void setIndex(int index) {
		if (this.project != null) {
			try {
				this.project.setIndex(index);
			} catch (RemoteException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		CenterController.this.refreshViews();
	}

	@Override
	public CenterViewManager getViewManager() {
		return (CenterViewManager) super.getViewManager();
	}

}
