/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9715 $
 * $Date: 2020-03-19 10:01:15 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.symspace.gui.utils;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Frame;

import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.SwingUtilities;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;

/**
 * 
 * @author Gnther Weidenholzer
 * @author Werner Koppelsttter
 */
public class ProgressBar extends JDialog {

	JProgressBar progressBar = null;
	JLabel statusLabel = null;
	int max = -1;

	// private class ColorMap {
	//
	// Color c1, c2;
	//
	// ColorMap(Color c1, Color c2) {
	// this.c1 = c1;
	// this.c2 = c2;
	// }
	//
	// Color getColor(float x) {
	//
	// float[] c1_ = this.c1.getRGBComponents(new float[4]);
	// float[] c2_ = this.c2.getRGBComponents(new float[4]);
	//
	// return new Color(c1_[0] + (x * (c2_[0] - c1_[0])),
	// c1_[1] + (x * (c2_[1] - c1_[1])),
	// c1_[2] + (x * (c2_[2] - c1_[2])));
	// }
	// }

	// ColorMap colmap = new ColorMap(Color.red, Color.green);

	/**
	 * 
	 * @param title
	 *            Title of progress bar
	 * @param max
	 */
	public ProgressBar(Frame frame, String title, int max) {
		this(frame, title);
		this.setMax(max);
	}

	public ProgressBar(Frame frame, String title) {
		super(frame);
		this.setTitle(title);
		this.setUndecorated(true);
		this.setLayout(new BorderLayout());

		JPanel mainPanel = new JPanel();
		mainPanel.setLayout(new BorderLayout());
		mainPanel.setBorder(new EtchedBorder());
		this.add(mainPanel, BorderLayout.CENTER);

		// Make a Progress Bar
		JPanel panel = new JPanel(new BorderLayout());
		panel.setBorder(new EmptyBorder(10, 10, 0, 10));
		mainPanel.add(panel, BorderLayout.CENTER);

		this.progressBar = new JProgressBar();
		this.progressBar.setMinimum(0);
		this.progressBar.setIndeterminate(true);
		this.progressBar.setStringPainted(false);
		panel.add(this.progressBar, BorderLayout.CENTER);

		this.statusLabel = new JLabel();
		this.statusLabel.setBorder(new EmptyBorder(5, 15, 5, 15));
		mainPanel.add(this.statusLabel, BorderLayout.SOUTH);

		this.setSize(450, 65);

		this.setValue(0, "Initialize");
		// this.setIconImages(Icons.getIcons());

	}

	public void show(Component owner) {
		this.setLocationRelativeTo(owner);
		this.setVisible(true);
	}

	public void setMax(int max) {
		if (max > 0) {
			this.max = max;
			this.progressBar.setMaximum(this.max);
			this.progressBar.setIndeterminate(false);
			this.progressBar.setStringPainted(true);
		}
	}

	/**
	 * Set current value
	 * 
	 * @param r
	 */
	public void setValue(final int r, final String status) {
		SwingUtilities.invokeLater(new Runnable() {

			@Override
			public void run() {
				ProgressBar.this.progressBar.setValue(r);
				if (ProgressBar.this.max > 0) {
					// ProgressBar.this.progressBar
					// .setForeground(ProgressBar.this.colmap.getColor(
					// (float) r / ProgressBar.this.max));
				}

				ProgressBar.this.statusLabel.setText(status);
			}
		});

	}

}
