/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9715 $
 * $Date: 2020-03-19 10:01:15 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.symspace.gui.views;

import javax.swing.JComponent;

/**
 * View for gui
 *
 * @author Werner Koppelsttter
 */
public abstract class View {

	/** View-manager */
	protected ViewManager manager;

	/** Parent view */
	protected View parent = null;

	/**
	 * Create view for view-manager
	 * 
	 * @param manager
	 */
	public View(ViewManager manager) {
		this.manager = manager;
	}

	/**
	 * Get parent view
	 * 
	 * @return Parent
	 */
	public View getParent() {
		return this.parent;
	}

	/**
	 * Set parent view
	 * 
	 * @param parent
	 *            Parent
	 */
	public void setParent(View parent) {
		this.parent = parent;
	}

	/**
	 * Get component to display
	 * 
	 * @return Component
	 */
	public abstract JComponent getComponent();

	/**
	 * Refresh view
	 */
	public abstract void refresh();

	/**
	 * Get view of view-item
	 * 
	 * @param item
	 *            View-item
	 * @return View
	 */
	public abstract View getView(ViewItem item);

	public abstract void setEnabled(boolean aFlag);
}
