/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9715 $
 * $Date: 2020-03-19 10:01:15 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.symspace.gui.views;

import java.util.LinkedList;

import javax.swing.JComponent;

import at.lcm.symspace.gui.Controller;

public abstract class ViewManager<V extends ViewItem> {

	protected LinkedList<V> viewItems = new LinkedList<V>();

	private Controller controller = null;

	private MenuBar menu;
	private ToolBar toolBar;
	private ToolBar statusBar;

	protected View views;

	protected boolean enabled = true;

	public ViewManager(Controller controller, MenuBar menu, ToolBar toolBar, ToolBar statusBar) {
		TabView tab;

		this.controller = controller;
		controller.setViewManager(this);

		this.menu = menu;
		this.toolBar = toolBar;
		this.statusBar = statusBar;

		this.refresh();
	}

	public MenuBar getMenu() {
		return this.menu;
	}

	public ToolBar getToolBar() {
		return this.toolBar;
	}

	public ToolBar getStatusBar() {
		return this.statusBar;
	}

	public Controller getController() {
		return this.controller;
	}

	public void setController(Controller controller) {
		this.controller = controller;
		controller.setViewManager(this);
	}

	public JComponent getComponent() {
		return this.views.getComponent();
	}

	public void refresh() {
		if (this.menu != null) {
			this.menu.refresh();
		}
		if (this.toolBar != null) {
			this.toolBar.refresh();
		}
		if (this.statusBar != null) {
			this.statusBar.refresh();
		}
		if (this.views != null) {
			this.views.refresh();
		}
	}

	public abstract void refresh(V item);

	public abstract JComponent getComponent(V item);

	public boolean isEnabled() {
		return this.enabled;
	}

	public void setEnabled(boolean aFlag) {
		this.enabled = aFlag;
		this.views.setEnabled(aFlag);
		this.menu.setEnabled(aFlag);
		this.toolBar.setEnabled(aFlag);
		this.statusBar.setEnabled(aFlag);
	}
}
