/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.docu;

import java.io.File;
import java.io.IOException;
import java.rmi.RemoteException;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;

import at.lcm.bu21.magopt.structure.ParamJavaFnc;
import at.lcm.bu21.magopt.structure.ParamMatlabFunction;
import at.lcm.bu21.magopt.structure.PortDesc;
import at.lcm.bu21.magopt.structure.Project;
import at.lcm.bu21.magopt.utils.MagOptConfig;
import at.lcm.bu21.magopt.utils.MatlabControl;
import matlabcontrol.MatlabInvocationException;
import matlabcontrol.MatlabProxy;

public class GenerateTableInfo {

	public static void main(String[] args) throws ZipException, IOException {
		ClassLoader cl;

		MagOptConfig.updateLib();
		cl = MagOptConfig.getClassLoader();
		System.out.println("#### JAVA ################################");
		GenerateTableInfo.createJavaFunctions(cl);
		System.out.println("#### MATLAB ################################");
		GenerateTableInfo.createMatlabFunctions(cl);
		System.out.println("#### Finished ################################");
		System.exit(0);
	}

	private static void createJavaFunctions(ClassLoader cl)
			throws ZipException, IOException {
		ZipFile zip;
		ZipEntry entry;
		Enumeration<? extends ZipEntry> en;
		String className, name;
		Class<?> c;

		for (File jar : MagOptConfig.getJavaPath()) {
			if (jar.getPath()
					.startsWith(Project.getLibraryPath() + File.separator)) {
				zip = new ZipFile(jar);
				en = zip.entries();
				while (en.hasMoreElements()) {
					entry = en.nextElement();
					className = entry.getName();
					if (className.toLowerCase().endsWith(".class")
							&& !className.toLowerCase().contains("$")) {
						name = className.substring(0, className.length() - 6)
								.replaceAll("/", ".");
						try {
							c = Class.forName(name, false, cl);
							if (ParamJavaFnc.class.isAssignableFrom(c)) {
								GenerateTableInfo.deklaredJavaFunction(
										(ParamJavaFnc) c.newInstance());
							}
						} catch (Throwable ex) {
						}
					}
				}
				zip.close();
			}
		}
	}

	private static void deklaredJavaFunction(ParamJavaFnc fnc) {

		// TODO
		System.out.println(fnc.getClass().getName());
		System.out.println("\t" + fnc.getClass().getProtectionDomain()
				.getCodeSource().getLocation().getFile());
		System.out.println("\t" + fnc.getDefaultDescription());
		System.out.println(
				"\t" + GenerateTableInfo.toPortDescString(fnc.importVars()));
		System.out.println(
				"\t" + GenerateTableInfo.toPortDescString(fnc.exportVars()));

	}

	private static void createMatlabFunctions(ClassLoader cl) {
		ParamMatlabFunction fnc;
		MatlabProxy proxy = null;
		Object[] ret;

		for (File folder : MagOptConfig.getMPath()) {
			if (folder.getPath()
					.startsWith(Project.getLibraryPath() + File.separator)) {
				for (File file : folder.listFiles()) {
					if (file.getName().startsWith("@")) {
						try {
							fnc = new ParamMatlabFunction("name", "desc",
									folder.getPath(),
									file.getName().substring(1));
							proxy = MatlabControl.getProxy(fnc.getClassPath());
							ret = proxy
									.returningEval("isa(" + fnc.getClassName()
											+ ", 'ParamMatlabFnc')", 1);
							if (((boolean[]) ret[0])[0]) {
								GenerateTableInfo.deklaredMatlabFunction(fnc,
										proxy);
							}
						} catch (Exception ex) {
						}
					}
				}
			}
		}

		try {
			if (proxy != null) {
				proxy.exit();
			}
		} catch (MatlabInvocationException e) {
		}
	}

	private static void deklaredMatlabFunction(ParamMatlabFunction fnc,
			MatlabProxy proxy)
			throws RemoteException, MatlabInvocationException {
		PortDesc[] list;

		// TODO
		System.out.println(fnc.getClassName());
		System.out.println("\t" + fnc.getClassPath());
		System.out.println("\t"
				+ GenerateTableInfo.getMatalbDefaultDescription(proxy, fnc));
		list = GenerateTableInfo.getMatalbImportVars(proxy, fnc);
		System.out.println("\t" + GenerateTableInfo.toPortDescString(list));
		list = GenerateTableInfo.getMatalbExportVars(proxy, fnc);
		System.out.println("\t" + GenerateTableInfo.toPortDescString(list));

	}

	private static String getMatalbDefaultDescription(MatlabProxy proxy,
			ParamMatlabFunction fnc)
			throws RemoteException, MatlabInvocationException {
		Object[] ret = null;

		ret = proxy.returningFeval("EvalFncMethode", 3, new Object[] {
				fnc.getRMIImpl().getClientImpl(), 1, "getDefaultDescription" });

		return (String) ret[2];
	}

	private static PortDesc[] getMatalbImportVars(MatlabProxy proxy,
			ParamMatlabFunction fnc)
			throws RemoteException, MatlabInvocationException {
		return GenerateTableInfo.getMatalbVars(proxy, fnc, "importVars");
	}

	private static PortDesc[] getMatalbExportVars(MatlabProxy proxy,
			ParamMatlabFunction fnc)
			throws RemoteException, MatlabInvocationException {
		return GenerateTableInfo.getMatalbVars(proxy, fnc, "exportVars");
	}

	private static PortDesc[] getMatalbVars(MatlabProxy proxy,
			ParamMatlabFunction fnc, String method)
			throws RemoteException, MatlabInvocationException {
		Object[] ret = null;
		PortDesc[] list = null;
		String[] names;

		ret = proxy.returningFeval("EvalFncMethode", 3,
				new Object[] { fnc.getRMIImpl().getClientImpl(), 1, method });

		if (ret[2] instanceof PortDesc[]) {
			list = (PortDesc[]) ret[2];
		} else if (ret[2] instanceof PortDesc) {
			list = new PortDesc[] { (PortDesc) ret[2] };
		} else if (ret[2] instanceof String[]) {
			names = (String[]) ret[2];
			list = new PortDesc[names.length];
			for (int k = 0; k < names.length; k++) {
				list[k] = new PortDesc(names[k], "", "", false);
			}
		} else if (ret[2] instanceof double[]) {
			list = new PortDesc[0];
		}

		return list;
	}

	private static String toPortDescString(PortDesc[] list) {
		StringBuffer buf;

		buf = new StringBuffer();
		buf.append("[");
		for (PortDesc pd : list) {
			buf.append(pd.getName());
			buf.append(", ");
		}
		if (buf.length() > 1) {
			buf.setLength(buf.length() - 2);
		}
		buf.append("]");

		return buf.toString();
	}
}
