/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9713 $
 * $Date: 2020-03-19 09:28:58 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.structure.converter;

import at.lcm.bu21.general.utils.XMLNode;

/**
 * Convert to V_0_21.
 * 
 * - Refill Parameter ID <br>
 * - Refill Dataset ID
 * 
 * @author Werner Koppelsttter
 */
public class Converter_21 {

	/**
	 * Convert the given node
	 * 
	 * @param node
	 *            Data to convert
	 * @return Changed data node
	 */
	static XMLNode convert(XMLNode node) {
		long nextid;

		if (node.getName().compareTo("Project") == 0) {

			// Refill parameter-id
			if (node.getNode(2).getName().compareTo("NextPID") == 0) {
				nextid = Long.parseLong(node.getNode(2).getText());
				nextid = Converter_21.refillPId(node, nextid);
				node.getNode(2).setText(Long.toString(nextid));
			}

			// Refill dataset-id
			Converter_21.refillDsId(node);
		}
		return node;
	}

	/**
	 * Refill parameter ID
	 * 
	 * @param node
	 *            Node to update
	 * @param nextID
	 *            Next id number
	 * @return Next id number
	 */
	private static long refillPId(XMLNode node, long nextID) {
		XMLNode item;

		if (node.getName().compareTo("Parameter") == 0) {

			item = node.getNode(0);
			if ((item.getText() == null) || item.getText().isEmpty()) {
				item.setText(Long.toString(nextID++, 36));
			}
		}

		// next Node
		for (XMLNode n : node.getNodes()) {
			nextID = Converter_21.refillPId(n, nextID);
		}

		return nextID;
	}

	/**
	 * Refill dataset ID
	 * 
	 * @param node
	 *            Node to update
	 */
	private static void refillDsId(XMLNode node) {
		int len;
		long nextid;
		XMLNode item, dsid;

		if ((node.getNode(0).getName().compareTo("Length") == 0)
				&& (node.getNode(3).getName().compareTo("NextDSID") == 0)
				&& (node.getNode(5).getName().compareTo("DSID") == 0)) {
			// get values
			len = Integer.parseInt(node.getNode(0).getText());
			nextid = Long.parseLong(node.getNode(3).getText());
			dsid = node.getNode(5);

			// change dsid
			while (dsid.getNodeSize() < len) {
				item = new XMLNode("item");
				item.setText(Long.toString(nextid++, 36));
				dsid.addNode(item);
			}

			// set value
			node.getNode(3).setText(Long.toString(nextid));
		}

	}
}
