/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.utils;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;

import at.lcm.bu21.magopt.structure.Parameter;

/**
 * Tranferable for parameters using the clipborad.
 * 
 * @author Werner Koppelsttter
 */
public class ParameterTransferable implements Transferable {
	// FIXME IParameter
	/** Used DataFlavor */
	public static DataFlavor _paramFlavor = new DataFlavor(Parameter.class,
			"ClipboardData");;

	/** Supported flavors */
	private static DataFlavor[] _supportedFlavors = new DataFlavor[] {
			ParameterTransferable._paramFlavor };

	/** Data in the clipboard */
	private Parameter _clipboardData;

	/**
	 * Create transferable for parameter
	 * 
	 * @param param
	 *            Parameter
	 */
	public ParameterTransferable(Parameter param) {
		this._clipboardData = param;
	}

	@Override
	public synchronized Object getTransferData(DataFlavor flavor)
			throws UnsupportedFlavorException, IOException {
		if (this.isDataFlavorSupported(flavor)) {
			return this._clipboardData;
		} else {
			throw new UnsupportedFlavorException(flavor);
		}
	}

	@Override
	public synchronized DataFlavor[] getTransferDataFlavors() {
		return ParameterTransferable._supportedFlavors;
	}

	@Override
	public boolean isDataFlavorSupported(DataFlavor flavor) {
		return flavor.equals(ParameterTransferable._paramFlavor);
	}
}
