/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.gui;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Hashtable;
import java.util.Map;

import magopt.gui.helpers.JProgressDialog;

/**
 * Runs Regenerate Process in a non-EDT Thread.
 * 
 * @author Aigner Markus
 * 
 */
public class RegenerateThread extends BaseThread implements
		PropertyChangeListener {
	private boolean mAllDS;

	protected RegenerateThread(MagOptGui gui, boolean allDS) {
		super(gui);
		this.mAllDS = allDS;

		this.addPropertyChangeListener(this);

		if (allDS == true) {
			this.mPD = new JProgressDialog(gui, "Regenerate Project",
					"Regenerating ...", 0, this.mGui.mProject.getLength());
		} else {
			this.mPD = new JProgressDialog(gui, "Regenerate Project",
					"Regenerating ...");
		}
	}

	@Override
	protected Void doInBackground() throws Exception {
		Map<String, Object> args = new Hashtable<String, Object>();
		args.put("RunSim", true);
		args.put("RemoteCompute", true);
		args.put("Optimize", true);

		if (this.mAllDS == true) {
			/* backup current index */
			int prevIndex = this.mGui.mProject.getIndex();
			for (int i = 0; i < this.mGui.mProject.getLength(); i++) {
				this.mGui.mProject.setIndex(i);
				this.mGui.mProject.refresh();
				this.mGui.mProject.regenerate(args);
				this.setProgress(i + 1);
			}

			/* set index to previous index */
			this.mGui.mProject.setIndex(prevIndex);
			this.mGui.mProject.refresh();
		} else {
			this.mGui.mProject.refresh();
			this.mGui.mProject.regenerate(args);
		}
		return null;
	}

	@Override
	public void propertyChange(PropertyChangeEvent pcEv) {
		if (pcEv.getPropertyName().equals("progress") == true) {
			this.mPD.setValue((Integer) (pcEv.getNewValue()));
		}
	}

	@Override
	protected void done() {
		this.mGui.updateTreeComponent();
		this.mGui.updateViewComponent();
		this.mGui.updateEventComponent();
		this.mPD.setVisible(false);
	}
}
