/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.gui.view.component;

import java.rmi.RemoteException;

import javax.swing.JTabbedPane;
import javax.swing.SwingConstants;

import at.lcm.bu21.magopt.structure.IComponent;
import at.lcm.bu21.magopt.structure.IParamContainer;
import at.lcm.bu21.magopt.structure.IParameter;
import magopt.gui.view.ParamBaseScroll;
import magopt.gui.view.ParamListener;

/**
 * Display content of component
 * 
 * @author Werner Koppelsttter
 */
public class ComponentView extends ParamBaseScroll {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** Tabbed pane for different views */
	private JTabbedPane _tab;

	/** Help browser of component */
	private ComponentHelpView _help;

	/** List of linked files */
	private LinkedFilesView _files;

	/**
	 * Create component view
	 */
	public ComponentView() {
		// this.setLayout(new BorderLayout());

		this._tab = new JTabbedPane(JTabbedPane.SCROLL_TAB_LAYOUT);
		this._tab.setTabPlacement(SwingConstants.BOTTOM);
		// this.add(this._tab, BorderLayout.CENTER);
		this.add(this._tab);

		// help browser
		this._help = new ComponentHelpView();
		this._tab.addTab("Help", this._help);
		this._help.addParamListener(new ParamListener() {
			@Override
			public void parameterChanged() {
				ComponentView.this.update();
			}
		});

		// list of linked files
		this._files = new LinkedFilesView();
		this._tab.addTab("Files", this._files);
		this._files.addParamListener(new ParamListener() {
			@Override
			public void parameterChanged() {
				ComponentView.this.update();
			}
		});
	}

	/**
	 * Sets new ParamComponent.
	 * 
	 * @param newComponent
	 *            - ParamComponent
	 */
	public void setComponent(IComponent newComponent) {
		this._help.setComponent(newComponent);
		this._files.setComponent(newComponent);
		this.update();
	}

	/**
	 * Set new ParamContainer
	 * 
	 * @param container
	 *            ParamContainer
	 */
	public void setContainer(IParameter container) {
		if (container instanceof IParamContainer) {
			try {
				this.setComponent(((IParamContainer) container).getComponent());
			} catch (RemoteException e) {
				this.setComponent(null);
			}
		} else {
			this.setComponent(null);
		}
	}

	@Override
	protected void update() {
		this._help.update();
		this._files.update();
	}

}
