/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.gui.view.port;

import java.awt.Component;
import java.rmi.RemoteException;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSplitPane;

import at.lcm.bu21.magopt.structure.DependDirection;
import at.lcm.bu21.magopt.structure.IParamFunction;
import at.lcm.bu21.magopt.structure.IParameter;
import magopt.gui.view.ParamBaseScroll;
import magopt.gui.view.ParamListener;

/**
 * Displays Depends specific Data:<br>
 * - Import Table on left Side<br>
 * - Export Table on right Side
 * 
 * @author Aigner Markus
 * @author Werner Koppelsttter
 */
public class ImportExportView extends ParamBaseScroll {
	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** ParamFunction to display */
	private IParamFunction paramFunction;

	/** Imports-table */
	// public for matlab gui
	public PortView imports;

	/** Export-table */
	// public for matlab gui
	public PortView exports;

	private JSplitPane mainPanel;

	/**
	 * Create and initialize view
	 */
	public ImportExportView() {
		this.mainPanel = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		this.mainPanel.setVisible(false);
		this.add(this.mainPanel);
		this.mainPanel.setDividerSize(5);
		this.mainPanel.setPreferredSize(this.getPreferredSize());
		this.mainPanel.setResizeWeight(0.5);
		this.mainPanel.setContinuousLayout(true);

		this.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

		// imports
		JPanel importPane = new JPanel();
		importPane.setLayout(new BoxLayout(importPane, BoxLayout.Y_AXIS));
		JLabel importLabel = new JLabel("I M P O R T S");
		importLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		importPane.add(importLabel);
		this.imports = new PortView();
		importPane.add(this.imports);
		this.mainPanel.setLeftComponent(importPane);
		this.imports.addParamListener(new ParamListener() {
			@Override
			public void parameterChanged() {
				ImportExportView.this.fireParamChanged();
			}
		});

		// exports
		JPanel exportPane = new JPanel();
		exportPane.setLayout(new BoxLayout(exportPane, BoxLayout.Y_AXIS));
		JLabel exportLabel = new JLabel("E X P O R T S");
		exportLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		exportPane.add(exportLabel);
		this.exports = new PortView();
		exportPane.add(this.exports);
		this.mainPanel.setRightComponent(exportPane);
		this.exports.addParamListener(new ParamListener() {
			@Override
			public void parameterChanged() {
				ImportExportView.this.fireParamChanged();
			}
		});

	}

	/**
	 * Sets new ParamFunction.
	 * 
	 * @param param
	 *            - ParamFunction
	 */
	public void setFunction(IParameter param) {
		if (param instanceof IParamFunction) {
			this.paramFunction = (IParamFunction) param;
		} else {
			this.paramFunction = null;
		}
		this.mainPanel.setVisible(this.paramFunction != null);
		this.update();
	}

	@Override
	protected void update() {
		try {
			if (this.paramFunction != null) {
				this.imports.setDepend(this.paramFunction.getDepend(), DependDirection.IMPORT);
				this.exports.setDepend(this.paramFunction.getDepend(), DependDirection.EXPORT);
			} else {
				this.imports.setDepend(null, DependDirection.IMPORT);
				this.exports.setDepend(null, DependDirection.EXPORT);
			}
		} catch (RemoteException e) {
			e.printStackTrace();
		}
	}
}
