/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.gui.view.pubinterface;

import java.awt.Component;
import java.awt.event.KeyEvent;

import javax.swing.BorderFactory;
import javax.swing.DefaultCellEditor;
import javax.swing.InputMap;
import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;

import at.lcm.bu21.magopt.structure.IParamContainer;
import magopt.gui.view.ParamBaseScroll;

/**
 * Displays public interface of container.
 * 
 * @author Werner Koppelsttter
 */
public class PubInterfaceView extends ParamBaseScroll {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** ParamContainer to display */
	private IParamContainer mParamContainer;

	/** Table to display container */
	// public for matlab gui
	public JTable table;

	/** Import-model */
	private PubInterfaceModel model;

	/**
	 * Create container-public-interface-view
	 */
	public PubInterfaceView() {
		this.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

		this.table = new JTable();
		this.model = new PubInterfaceModel();
		this.table.setModel(this.model);
		this.table.getTableHeader().setReorderingAllowed(false);
		this.table.setColumnSelectionAllowed(true);
		this.table.setRowSelectionAllowed(true);
		this.table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		this.table.setDefaultRenderer(Object.class, new PubInterfaceRenderer());
		// Select test by enter the editor
		this.table.setDefaultEditor(Object.class, new DefaultCellEditor(new JTextField()) {

			/** Serialization version number */
			private static final long serialVersionUID = 1L;

			@Override
			public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row,
					int column) {
				Component c;
				c = super.getTableCellEditorComponent(table, value, isSelected, row, column);
				((JTextField) c).selectAll();
				return c;
			}
		});
		// Change action of enter to action of tab (change to next right cell)
		InputMap im = this.table.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
		im.put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), im.get(KeyStroke.getKeyStroke(KeyEvent.VK_TAB, 0)));

		this.model.addTableModelListener(new TableModelListener() {
			@Override
			public void tableChanged(TableModelEvent e) {
				PubInterfaceView.this.fireParamChanged();
			}
		});
		this.add(this.table);

		this.table.getColumnModel().getColumn(PubInterfaceColumn.CHECK.getValue()).setMaxWidth(30);

	}

	/**
	 * Sets new ParamContainer.
	 * 
	 * @param newContainer
	 *            - ParamContainer
	 */
	public void setContainer(IParamContainer newContainer) {
		if (this.table.isEditing()) {
			this.table.getCellEditor().cancelCellEditing();
		}
		this.mParamContainer = newContainer;
		this.update();
	}

	@Override
	protected void update() {
		this.model.setContainer(this.mParamContainer);
	}
}
