/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.structure.database;

import java.io.Serializable;

/**
 * Database item
 */
public class DBItem implements Serializable {

	/** Serialization id */
	private static final long serialVersionUID = 1L;

	/** ID of item */
	private long id;

	/** Name of item */
	private String name;

	/**
	 * Construct a emtpy dataset-item
	 */
	public DBItem() {
		this(-1, "---");
	}

	/**
	 * Construct dataset-item
	 * 
	 * @param id
	 *            ID of item
	 * @param name
	 *            Name of item
	 */
	public DBItem(long id, String name) {
		this.id = id;
		this.name = name;
	}

	/**
	 * Get clone of this class
	 * 
	 * @return Clone of object
	 */
	public DBItem clone() {
		DBItem clone = new DBItem();
		clone.id = this.id;
		clone.name = this.name;
		return clone;
	}

	/**
	 * Get ID of item
	 * 
	 * @return ID
	 */
	public long getID() {
		return this.id;
	}

	/**
	 * Get name if item
	 * 
	 * @return Name
	 */
	public String getName() {
		return this.name;
	}

	/**
	 * Get the representing string
	 * 
	 * @return Representing string
	 */
	public String toString() {
		return this.name;
	}

	/**
	 * Check if an object is equal to this object
	 * 
	 * @param obj
	 *            Object to compare
	 * @return Is this equal to other object
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		DBItem other = (DBItem) obj;
		if (id != other.id)
			return false;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		return true;
	}

}
