"""
Created on 03.05.2017

@author: Siegfried Silber
"""

import json
from symspace.wrapper.symwrapper import SymProject


def writeJSON(prj: SymProject, fname_comp: str, help_dir: str):
    """
    Write JSON File

    Parameters
    ----------
    prj : SymProject
    fname_comp : string
        File name of the Component.

    """
    inc_lst = ['Info', 'Files', 'Docu', 'Filter']
    prj_dict = prj.toDict()
    json_dict = {}

    for item in inc_lst:
        if item in prj_dict:
            json_dict[item] = prj_dict[item]

    # Infos for database
    types_enum = {'OTHER': '0', 'PROPERTY': '1', 'PARAMETER': '2', 'TAG': '3'}

    # Redefine provides
    ruleCond = []
    if 'provides' in json_dict['Filter']:
        provides = json_dict['Filter']['provides']

        for key in provides:
            if isinstance(provides[key], dict):
                cond = provides[key]
                if 'type' in cond:
                    cond['type'] = types_enum[cond['type']]
                cond.pop('__error__', None)
                rule = {'logicalOperator': 'OR',
                        'ruleConditions': [cond],
                        'name': cond['value']}
                ruleCond.append(rule)

        provisions = {'logicalOperator': 'OR', 'rules': ruleCond}
        json_dict['Filter']['provisions'] = provisions

        ''' Remove provides from json_dict '''
        json_dict['Filter'].pop('provides', None)

    ''' Redefine requires '''
    rule = []
    if 'requires' in json_dict['Filter']:
        requires = json_dict['Filter']['requires']

        if 'AND' in requires:
            operator = 'AND'
            subop = requires['AND']
        elif 'OR' in requires:
            operator = 'OR'
            subop = requires['OR']

        for op in subop:
            innerop = subop[op]
            if not isinstance(innerop, dict):
                continue

            if op.startswith('AND'):
                log_operator = 'AND'
            elif op.startswith('OR'):
                log_operator = 'OR'

            ruleCond = []
            ruleInfo = {'logicalOperator': log_operator}

            displayName = op.replace(log_operator, '')
            if displayName.startswith('_'):
                displayName = displayName[1:]
            ruleInfo['name'] = '"{}"'.format(displayName)

            for key in innerop:
                if isinstance(innerop[key], dict):
                    cond = innerop[key]
                    if 'type' in cond:
                        cond['type'] = types_enum[cond['type']]
                    cond.pop('__error__', None)
                    ruleCond.append(cond)
                else:
                    if key == 'min':
                        ruleInfo['min'] = '{}'.format(int(float(innerop[key])))
                    elif key == 'max':
                        ruleInfo['max'] = '{}'.format(int(float(innerop[key])))
                    elif key == 'displayName':
                        ruleInfo['name'] = innerop[key]
                    elif key == '__error__':
                        pass
                    else:
                        raise Exception('Key Error in {}'.format(innerop))
            ruleInfo['ruleConditions'] = ruleCond
            rule.append(ruleInfo)

        requirements = {'logicalOperator': operator, 'rules': rule}
        json_dict['Filter']['requirements'] = requirements

        ''' Remove requres from json_dict '''
        json_dict['Filter'].pop('requires', None)

    '''Add help directory'''
    json_dict['Filter']['HelpDir'] = help_dir

    jsonstr = json.dumps(json_dict)
    fname = fname_comp.replace('.mopc', '.json')
    fh = open(fname, "w+")
    fh.write(jsonstr)
    fh.close()


def readInterface(params: list) -> list:
    """
    Read parameters of the Component function

    Parameters
    ----------
    params : list
        List of SymDependItem
        SyMSpace import or export

    Returns
    -------
    par_lst : list
        List of parameters
    """
    par_lst = []
    for param in params:
        item = {}
        # if not (param.getParamName() == ''):
        #     ''' get parameters from the fields linked to the function interface'''
        #     item['name'] = param.getParamName()
        #     if param.getHandle() is None:
        #         item['description'] = param.javaObj.getVarDesc().getDescription()
        #         item['info'] = 'external'
        #         item['comment'] = ''
        #     else:
        #         item['description'] = param.getHandle().getDescription()
        #         item['info'] = ''
        #         if param.getHandle().getComment().replace("'''", "").strip().find('#') == 0:
        #             item['comment'] = param.getHandle().getComment().replace("'''", "").replace("#", "").strip()
        # else:
        #     item['name'] = param.getVarName()
        #     item['description'] = param.javaObj.getVarDesc().getDescription()
        #     item['info'] = 'optional'
        #     item['comment'] = ''

        ''' get parameters from the function interface'''
        ports_desc = param.javaObj.getVarDesc()
        name = ports_desc.getName()
        description = ports_desc.getDescription()
        unit = ports_desc.getUnit()
        optional = ports_desc.getOptional()
        dimension = ports_desc.getDimension()
        if param.getHandle() is None:
            info = 'external'
        else:
            info = ''

        item['name'] = name
        item['description'] = description
        item['unit'] = unit
        item['optional'] = optional
        item['dimension'] = dimension
        # ToDo: do we need info/comment?
        item['info'] = info
        item['comment'] = ''

        par_lst.append(item)
    return par_lst
