#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 9639 $
$LastChangedDate:: 2020-02-12 13:38:34 +0100#$

This file is part of SyMSpace.

Copyright (c) 2018, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""

import sys
import os
import subprocess
from symspace.generic_gui_qt import GUIFormThread
from symspace.wrapper.symwrapper import SymPythonFunction, SymContainer


class NewPythonFncGUI:

    def __init__(self, param: SymContainer, dirpath: str):
        self.dirpath = dirpath
        self.param = param
        self.name = 'TemplateFunction'
        self.description = 'Function Description'
        self.open_script = False
        self.open_dir = False
        self.create_test = False
        self.fnc = None

    def start(self):
        gui_elements = self.getGUIElements()
        title = 'Create new Python Function'
        gui = GUIFormThread(title, gui_elements)
        gui.start()

        if gui.accept:
            self.updateMembers(**gui.result)
            self.fnc = SymPythonFunction.new(self.param, self.name, self.description, self.dirpath, self.create_test)

            if self.open_dir:
                self.openDirectory()

            '''Open new function in editor'''
            if self.open_script:
                self.openScript()

    def updateMembers(self, *args, **kwargs):
        """
        Update member variables with GUI results

        Parameters
        ----------
        """

        self.name = kwargs['Name']
        self.description = kwargs['Description']
        if kwargs['Select directory'] != '':
            self.dirpath = kwargs['Select directory']
        else:
            self.dirpath = os.path.join(self.param.getRoot().getDirectory(), 'src')

        self.open_script = kwargs['Open in editor']
        self.open_dir = kwargs['Open folder']
        self.create_test = kwargs['Create test']

    def getGUIElements(self):
        """
        GUI structure
        """

        gui_elements = [('Name', 'line', self.name),
                        ('Description', 'line', self.description),
                        ('Select directory', 'browse', self.dirpath),
                        ('Open in editor', 'checkbox', self.open_script),
                        ('Open folder', 'checkbox', self.open_dir),
                        ('Create test', 'checkbox', self.create_test)
                        ]
        return gui_elements

    def openDirectory(self):
        """Open the function directory in the system file manager"""
        try:
            if sys.platform == "win32":
                '''Windows'''
                subprocess.call("explorer " + self.dirpath)
                # subprocess.call("cmd /c start " + self.fnc_path)
            elif sys.platform == "linux" or sys.platform == "linux2":
                '''linux'''
                raise NotImplementedError
            elif sys.platform == "darwin":
                ''' OS X'''
                raise NotImplementedError
        except Exception as e:
            print(e)
            print('Unable to open dir')

    def openScript(self):
        """ Open new function in editor

        Returns
        -------

        """
        ''''''
        if self.open_script:
            try:
                self.fnc.edit()
            except Exception as e:
                print(e)
                print('Failed to open!')
