#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.5

$LastChangedRevision: 9792 $
$LastChangedDate:: 2020-04-22 09:50:28 +0200#$

This file is part of SyMSpace.

Copyright (c) 2017, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.

Comparator.plotParamData(CurProject['PyPostprocess.recup_rate'])

from symspace.wrapper.pyplot import *
"""

try:
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.general.plot import LinkPlot
    # from at.lcm.bu21.general.plot import Plot
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.general.plot import Plot2Area
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.general.plot import Plot2Bar
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.general.plot import Plot2ContourFilled
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.general.plot import Plot2D
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.general.plot import Plot2Hist
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.general.plot import Plot2Line
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.general.plot import Plot2Polar
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.general.plot import Plot2Scatter
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.general.plot import Plot2Spline
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.general.plot import Plot2XY
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.general.plot import Plot3ContourFilled
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.general.plot import Plot3Scatter
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.general.plot import Plot3Surface
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.general.plot import SubPlot
    # noinspection PyUnresolvedReferences
    from java.awt import GridLayout
    # noinspection PyUnresolvedReferences
    from java.awt import Dimension
    # noinspection PyUnresolvedReferences
    from javax.swing import JPanel
    # noinspection PyUnresolvedReferences
    from java.io import File
    from lcm.tdata_interface.pyjdata_interface import PyJDataInterface

except Exception as e:
    print(e)

import numpy as np
from abc import ABCMeta


class SymPanel:
    """
    Extended wrapper for JPanel class
    """

    def __init__(self, rows=1, cols=1):
        """
        Initialize panel with defined grid layout (rows, cols)
        Parameters
        ----------
        rows : int (Optional)
            Number of rows (Default: 1)
        cols : int (Optional)
            Number of columns (Default: 1)
        """
        self.jpanel = JPanel()
        self.setLayout(rows, cols)

    def getPanel(self):
        """
        Get the current panel

        Returns
        -------
        panel : JPanel
        """
        return self.jpanel

    def setLayout(self, rows, cols):
        """
        Set the grid layout of the panel

        Parameters
        ----------
        rows : int
            Number of rows
        cols : int
            Number of columns

        """
        self.jpanel.setLayout(GridLayout(rows, cols))

    def addPlot(self, sym_plot, index=None):
        """
        Add a plot into the panel

        Parameters
        ----------
        sym_plot : SymPlot
            The SymPlot to be added
        index : int (Optional)
            The position at which to insert the component.
            The index starts with 0.
            Use -1 to append the component to the end.

        """
        if index is None:
            self.jpanel.add(sym_plot.getPanel())
        else:
            self.jpanel.add(sym_plot.getPanel(), int(index))


class SymLinkPlot:
    """
    Link selection of different plots
    """

    def __init__(self):
        self.link = LinkPlot()

    def addPlot(self, symplot):
        self.link.addPlot(symplot.hfig)


class SymPlot(metaclass=ABCMeta):
    """
    Basic class of plots
    """
    hfig = None

    def getPanel(self):
        """
        Get the current panel

        Returns
        -------
        panel : JPanel
        """
        panel = self.hfig.getPanel()
        '''panel shall automatically resize to frame extends'''
        panel.setPreferredSize(Dimension())
        return panel

    def clearSignals(self):
        """
        Remove all signals
        """
        self.hfig.clearSignals()

    def getColorDirection(self):
        """
        Get color-direction of colorbar(standard or inverse)

        Returns
        -------
        colorDirection : bool

        """
        return self.hfig.getColorDirection()

    def getColorLevels(self):
        """
        List color-levels of colorbar

        Returns
        -------
        colorLevels : float

        """
        return self.hfig.getColorLevels()

    def getNumSignals(self):
        """
        Get number of signals

        Returns
        -------
        numSignals : int

        """
        return self.hfig.getNumSignals()

    def show(self, title=''):
        """
        Show plot in a frame

        Parameters
        ----------
        title : string, optional

        """
        if len(title) == 0:
            self.hfig.show()
        else:
            self.hfig.show(title)

    def getTitle(self):
        """
        Returns the chart title

        Returns
        -------
        title : str

        """
        return self.hfig.getTitle()

    def setTitle(self, title):
        """
        Sets the chart title

        Parameters
        ----------
        title : string

        """
        self.hfig.setTitle(title)

    def reset(self):
        """
        Resets the whole panel

        """
        self.hfig.reset()

    def getXLabel(self):
        """
        Returns x-Axis label

        Returns
        -------
        label : string

        """

        return self.hfig.getXLabel()

    def setXLabel(self, label):
        """
        Sets the x label

        Parameters
        ----------
        label : string

        """
        self.hfig.setXLabel(label)

    def getYLabel(self):
        """
        Returns the y-Axis label

        Returns
        -------
        label : string

        """
        return self.hfig.getYLabel()

    def setYLabel(self, label):
        """
        Sets the y label

        Parameters
        ----------
        label : string

        """
        self.hfig.setYLabel(label)

    def getZLabel(self):
        """
        Returns the z-Axis label

        Returns
        -------
        label : string

        """
        return self.hfig.getZLabel()

    def setZLabel(self, label):
        """
        Sets the Z Label

        Parameters
        ----------
        label : string

        """
        self.hfig.setZLabel(label)

    def save(self, file_path, width=None, height=None):
        """
        Exports the whole Plot to a file depending on the file Type

        """
        if width is None:
            self.hfig.save(file_path)
        else:
            self.hfig.save(file_path, width, height)

    def getSignal(self, index):
        """
        Get signal of index

        Parameters
        ----------
        index : int
            Index of signal

        Returns
        -------
        signal : Signal

        """  # Todo: add class signal
        return self.hfig.getSignal(index)

    def removeSignal(self, index):
        """
        Remove Signal

        Parameters
        ----------
        index : int

        Returns
        -------
        signal : Signal

        """
        return self.hfig.removeSignal(index)


class SymSubPlot(SymPlot):
    """
    Class for Creating SubPlots
    """

    def __init__(self, row, col):
        self.hfig = SubPlot(row, col)
        self.__sub_plots = np.empty((row, col), np.dtype(object))

    def getSize(self):
        """
        Return the size of the SubPlot (row,col)

        Returns
        -------
        tuple
        """
        return self.hfig.getSize()

    # def getPlot(self, row, col):
    #     """
    #     Get plot of positon
    #
    #     Parameters
    #     ----------
    #     row: int
    #     col : int
    #
    #     Returns
    #     -------
    #
    #     """
    #     hfig = self.hfig.getPlot(row, col)
    #     self.__sub_plots[row, col].hfig = hfig
    #     return self.__sub_plots[row, col]

    # def setPlot(self, row, col, symplot):
    #     """
    #     Sets one particular Plot
    #
    #     Parameters
    #     ----------
    #     row: int
    #     col: int
    #     symplot: SymPlot
    #
    #     """
    #     if hasattr(symplot, 'hfig'):
    #         self.hfig.setPlot(row, col, symplot.hfig)
    #         self.__sub_plots[row, col] = symplot

    # def setPlots(self, symplots):
    #     """
    #     Sets the plot array which contains all plots
    #
    #     Parameters
    #     ----------
    #     symplots : nd.array (dtype=object)
    #
    #     """
    #     if not isinstance(symplots, np.ndarray):
    #         raise TypeError('Expected np.ndarray got ' + str(type(symplots)) + ' instead')
    #     if not symplots.dtype == object:
    #         raise TypeError('Expected dtype=object got ' + str(symplots.dtype) + ' instead')
    #     if not symplots.ndim == 2:
    #         raise Exception('Expected ndim=2 got ' + str(symplots.ndim) + ' instead')
    #
    #     n_rows, n_cols = self.getSize()
    #     n_dim1, n_dim2 = symplots.shape
    #
    #     if n_rows != n_dim1:
    #         raise Exception('Expected n_rows=' + str(n_rows) + ' got ' + str(n_dim1) + ' instead')
    #     if n_cols != n_dim2:
    #         raise Exception('Expected n_cols=' + str(n_cols) + ' got ' + str(n_dim2) + ' instead')
    #
    #     for row in range(n_rows):
    #         for col in range(n_cols):
    #             self.setPlot(row, col, symplots[row, col])

    def setSize(self, row, col):
        """
        Sets the Size of the SubPlot

        Parameters
        ----------
        row: int
        col: int

        """
        self.hfig.setSize(row, col)

    def __setitem__(self, indices, item):
        row, col = indices
        self.hfig.setPlot(row, col, item.hfig)
        self.__sub_plots[row, col] = item

    def __getitem__(self, indices):
        row, col = indices
        return self.__sub_plots[row, col]


class SymPlot2D(SymPlot, metaclass=ABCMeta):
    """
    Base type of TData plotting classes
    """

    def clearXLimit(self):
        """

        Clear the x limit

        """
        self.hfig.clearXLimit()

    def clearYLimit(self):
        """

        Clear the y limit

        """
        self.hfig.clearYLimit()

    def clearZLimit(self):
        """

        Clear the z limit

        """
        self.hfig.clearZLimit()

    def getLegend(self):
        """

        Returns the Legend visibility

        Returns
        -------
        visibility : bool

        """
        return self.hfig.getLegend()

    def setLegend(self, b):
        """
        Turns the Legend on/off

        Parameters
        ----------
        b : bool

        """
        self.hfig.setLegend(b)

    def reset(self):
        """
        Resets the whole panel
        """
        self.hfig.reset()

    def zoom(self):
        """
        This Methode zooms the whole plot to the right perspective
        """
        self.hfig.zoom()

    def setXLimit(self, min, max):
        """
        Sets the x limit

        Parameters
        ----------
        min : double
        max : double

        """
        self.hfig.setXLimit(min, max)

    def setYLimit(self, min, max):
        """
        Sets the y limit

        Parameters
        ----------
        min : double
        max : double

        """
        self.hfig.setYLimit(min, max)

        # def setColorTransparency(self, alpha):
        #     self.hfig.setColorTransparency(alpha)


class SymPlot2XY(SymPlot2D, metaclass=ABCMeta):
    """
    Class for plotting signals
    """

    def addSignal(self, name, *args, **kwargs):
        """
        Add a signal to the plot

        Parameters
        ----------
        name : string
            Name of signal
        xSignal : np.ndarray or PyTPeriodicMatrix of size (1,1)
        ySignal : np.ndarray or PyTPeriodicMatrix of size (1,1)
        nPoints : int

        linewidth : float
            e.g. addSignal('Name', x, y, linewidth=1.5)

        """
        if len(args) == 1:
            ySignal = args[0]
            if isinstance(ySignal, list):
                ySignal = np.asarray(ySignal)
            self.hfig.addSignal(name, PyJDataInterface.convert_Py2J(ySignal))

        elif len(args) == 2:
            xSignal = args[0]
            ySignal = args[1]
            if isinstance(xSignal, list):
                xSignal = np.asarray(xSignal)
            if isinstance(ySignal, list):
                ySignal = np.asarray(ySignal)
            self.hfig.addSignal(name, PyJDataInterface.convert_Py2J(xSignal), PyJDataInterface.convert_Py2J(ySignal))

        elif len(args) == 3:
            xSignal = args[0]
            ySignal = args[1]
            nPoints = args[2]
            if isinstance(xSignal, list):
                xSignal = np.asarray(xSignal)
            if isinstance(ySignal, list):
                ySignal = np.asarray(ySignal)
            self.hfig.addSignal(name, PyJDataInterface.convert_Py2J(xSignal), PyJDataInterface.convert_Py2J(ySignal),
                                nPoints)

        else:
            raise ValueError('Wrong number of parameters')

        if 'linewidth' in kwargs:
            self.setLineWidth(self.getNumSignals() - 1, kwargs['linewidth'])

    def setLineWidth(self, signal, linewidth):
        """
        Set line width of signal

        Parameters
        ----------
        signal : int
            Number of signal

        linewidth : float
            Width of line

        """
        style = self.getSignal(signal).getStyle()
        style.setLineWidth(linewidth)
        self.reset()


class SymPlot2Hist(SymPlot2XY):
    """
    Histogram plot. Draw distribution of y-values.
    """

    def __init__(self):
        self.hfig = Plot2Hist()

    def getNumIntervals(self):
        """
        Get number of intervals

        Returns
        -------
        number : int

        """
        return self.hfig.getNumIntervals()

    def setNumIntervals(self, numIntervals):
        """
        Set number of intervals

        Parameters
        ----------
        numIntervals : int

        """
        self.hfig.setNumIntervals(numIntervals)


class SymPlot2Area(SymPlot2XY):
    """
    Area plot
    """

    def __init__(self):
        self.hfig = Plot2Area()


class SymPlot2ContourFilled(SymPlot2XY):
    """
    Area plot
    """

    def __init__(self):
        self.hfig = Plot2ContourFilled()


class SymPlot2Line(SymPlot2XY):
    """
    Line plot
    """

    def __init__(self):
        self.hfig = Plot2Line()


class SymPlot2Bar(SymPlot2XY):
    """
    Bar plot
    """

    def __init__(self):
        self.hfig = Plot2Bar()


class SymPlot2Scatter(SymPlot2XY):
    """
    Scatter  plot
    """

    def __init__(self):
        self.hfig = Plot2Scatter()


class SymPlot2Polar(SymPlot2D):
    """
    Polar  plot
    """

    def __init__(self):
        self.hfig = Plot2Polar()

    def addSignal(self, name, *args):
        """
        Add a signal to the plot

        Parameters
        ----------
        name : string
            Name of signal
        xSignal : np.ndarray or PyTPeriodicMatrix of size (1,1)
        ySignal : np.ndarray or PyTPeriodicMatrix of size (1,1)
        nPoints : int

        """
        if len(args) == 1:
            ySignal = args[0]
            if isinstance(ySignal, list):
                ySignal = np.asarray(ySignal)
            self.hfig.addSignal(name, PyJDataInterface.convert_Py2J(ySignal))

        elif len(args) == 2:
            xSignal = args[0]
            ySignal = args[1]
            if isinstance(xSignal, list):
                xSignal = np.asarray(xSignal)
            if isinstance(ySignal, list):
                ySignal = np.asarray(ySignal)
            self.hfig.addSignal(name, PyJDataInterface.convert_Py2J(xSignal), PyJDataInterface.convert_Py2J(ySignal))

        elif len(args) == 3:
            xSignal = args[0]
            ySignal = args[1]
            nPoints = args[2]
            if isinstance(xSignal, list):
                xSignal = np.asarray(xSignal)
            if isinstance(ySignal, list):
                ySignal = np.asarray(ySignal)
            self.hfig.addSignal(name, PyJDataInterface.convert_Py2J(xSignal), PyJDataInterface.convert_Py2J(ySignal),
                                nPoints)

        else:
            raise ValueError('Wrong number of parameters')


class SymPlot2Spline(SymPlot2XY):
    """
    Spline plot
    """

    def __init__(self):
        self.hfig = Plot2Spline()


class SymPlot3D(SymPlot, metaclass=ABCMeta):
    """
    Class for plotting 3D signals
    """

    def getColorLegend(self):
        """

        Returns the Legend visibility

        Returns
        -------
        visibility : bool

        """
        return self.hfig.getColorLegend()

    def getViewPoint(self):
        """
        Get the view-point

        Returns
        -------
        (x, y, z) : list
        """
        return list(self.hfig.getViewPoint())

    def setColorLegend(self, b):
        """
        Turns the Legend on/off

        Parameters
        ----------
        b : bool

        """
        self.hfig.setColorLegend(b)

    def setViewPoint(self, x, y, z):
        """
        Set the view-point

        Parameters
        ----------
        x : float
        y : float
        z : float

        """
        self.hfig.setViewPoint(x, y, z)

    def addSignal(self, name, *args):
        """
        Add a signal to the plot

        Parameters
        ----------
        name : string
            Name of signal
        xSignal : np.ndarray
        ySignal : np.ndarray
        zSignal : np.ndarray

        """
        if len(args) == 1:
            zSignal = args[0]

            if zSignal.ndim != 2:
                raise Exception('Dimension of z: 2 != ' + str(zSignal.ndim))

            if isinstance(zSignal, list):
                zSignal = np.asarray(zSignal)

            z = PyJDataInterface.convert_Py2J(zSignal)

            self.hfig.addSignal(name, z)

        elif len(args) == 3:
            xSignal = args[0]
            ySignal = args[1]
            zSignal = args[2]

            if zSignal.ndim != 2:
                raise Exception('Dimension of z: 2 != ' + str(zSignal.ndim))

            if isinstance(xSignal, list):
                xSignal = np.asarray(xSignal)
            if isinstance(ySignal, list):
                ySignal = np.asarray(ySignal)
            if isinstance(zSignal, list):
                zSignal = np.asarray(zSignal)

            xSignal = xSignal.flatten()
            ySignal = ySignal.flatten()

            if zSignal.shape != (xSignal.size, ySignal.size):
                raise Exception(
                    'Shape z: ' + str(zSignal.shape) + ' does not match (x, y): ' + str((xSignal.size, ySignal.size)))
            zSignal = zSignal.transpose()

            x = PyJDataInterface.convert_Py2J(xSignal)
            y = PyJDataInterface.convert_Py2J(ySignal)
            z = PyJDataInterface.convert_Py2J(zSignal)

            self.hfig.addSignal(name, x, y, z)
        else:
            raise ValueError('Wrong number of parameters')


class SymPlot3Scatter(SymPlot3D):
    """
    3D Surface Plot
    """

    def __init__(self):
        self.hfig = Plot3Scatter()

    def addSignal(self, name, *args):
        """
        Add a signal to the plot

        Parameters
        ----------
        name : string
            Name of signal
        xSignal : np.ndarray
        ySignal : np.ndarray
        zSignal : np.ndarray

        """
        try:
            xSignal = args[0]
            ySignal = args[1]
            zSignal = args[2]

            if isinstance(xSignal, list):
                xSignal = np.asarray(xSignal)
            if isinstance(ySignal, list):
                ySignal = np.asarray(ySignal)
            if isinstance(zSignal, list):
                zSignal = np.asarray(zSignal)

            xSignal = xSignal.flatten()
            ySignal = ySignal.flatten()
            zSignal = zSignal.flatten()

            x = PyJDataInterface.convert_Py2J(xSignal)
            y = PyJDataInterface.convert_Py2J(ySignal)
            z = PyJDataInterface.convert_Py2J(zSignal)

            self.hfig.addSignal(name, x, y, z)
        except:
            super().addSignal(name, *args)


class SymPlot3Surface(SymPlot3D):
    """
    3D Surface Plot
    """

    def __init__(self):
        self.hfig = Plot3Surface()


class SymPlot3ContourFilled(SymPlot3D):
    """
    3D filled contour plot
    """

    def __init__(self):
        self.hfig = Plot3ContourFilled()


if __name__ == "__main__":

    x1 = np.arange(0, 10, 1)
    y1 = np.sin(x1)

    x2 = np.linspace(0, 2 * np.pi)
    y2 = np.sin(x2)

    x3d = np.array([0, 3])
    y3d = np.array([0, 2])
    z3d = np.array([[0, 2], [1, 2]])

    x3ds = np.array([1, 2, 3, 4, 5, 6])
    y3ds = np.array([1, 2, 3, 4, 5, 6])
    z3ds = np.array([1, 2, 3, 4, 5, 6]) * 100

    '''2D'''
    area2D = SymPlot2Area()
    area2D.addSignal('area2D', x1, y1)
    area2D.setTitle('area2D')
    # area2D.show()

    bar2D = SymPlot2Bar()
    bar2D.addSignal('bar2D', x1, y1)
    bar2D.setTitle('bar2D')
    # bar2D.show()

    contour_filled2D = SymPlot2ContourFilled()
    contour_filled2D.addSignal('contour_filled2D', x3d, y3d)
    contour_filled2D.setTitle('contour_filled2D')
    # contour_filled2D.show()

    hist2D = SymPlot2Hist()
    hist2D.addSignal('hist2D', x1, y1)
    hist2D.setTitle('hist2D')
    # hist2D.show()

    line2D = SymPlot2Line()
    line2D.addSignal('line2D', x1, y1)
    line2D.setTitle('line2D')
    # line2D.show()

    polar2D = SymPlot2Polar()
    polar2D.addSignal('polar2D', x2, y2)
    polar2D.setTitle('polar2D')
    # polar2D.show()

    scatter2D = SymPlot2Scatter()
    scatter2D.addSignal('scatter2D', x1, y1)
    scatter2D.setTitle('scatter2D')
    # scatter2D.show()

    spline2D = SymPlot2Spline()
    spline2D.addSignal('spline2D', x1, y1)
    spline2D.setTitle('spline2D')
    # spline2D.show()

    '''3D'''
    contour_filled3D = SymPlot3ContourFilled()
    contour_filled3D.addSignal('contour_filled3D', x3d, y3d, z3d)
    contour_filled3D.setTitle('contour_filled3D')
    contour_filled3D.setViewPoint(0, 0, 0)
    # contour_filled3D.show()

    surface3D = SymPlot3Surface()
    surface3D.addSignal('surface3D', x3d, y3d, z3d)
    surface3D.setTitle('surface3D')
    surface3D.setViewPoint(0, 0, 0)
    # surface3D.show()

    scatter3D = SymPlot3Scatter()
    scatter3D.addSignal('scatter3D', x3ds, y3ds, z3ds)
    scatter3D.setTitle('scatter3D')
    scatter3D.setViewPoint(0, 0, 0)
    # scatter3D.show()

    n_rows = 8
    n_cols = 2

    sym_subplot = SymSubPlot(n_rows, n_cols)
    sym_subplot[0, 0] = area2D
    sym_subplot[1, 0] = bar2D
    sym_subplot[2, 0] = contour_filled2D
    sym_subplot[3, 0] = hist2D
    sym_subplot[4, 0] = line2D
    sym_subplot[5, 0] = polar2D
    sym_subplot[6, 0] = scatter2D
    sym_subplot[7, 0] = spline2D
    sym_subplot[0, 1] = contour_filled3D
    sym_subplot[1, 1] = surface3D
    sym_subplot[2, 1] = scatter3D
    sym_subplot.show()
