/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9720 $
 * $Date: 2020-03-19 15:59:49 +0100 (Thu, 19 Mar 2020) $
 */
package jobmanager.server;

import java.io.FileOutputStream;

import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamWriter;

import at.lcm.bu21.magopt.structure.Project;
import at.lcm.bu21.magopt.structure.event.Event;
import at.lcm.bu21.magopt.structure.event.EventType;

/**
 * Run a job
 * 
 * @author Werner Koppelsttter
 */
public class JobStarter {

	/**
	 * Run a job with the given parameter
	 * 
	 * @param args
	 *            Command-line parameter for job
	 * @throws Exception
	 * @throws ClassNotFoundException
	 * @throws IllegalAccessException
	 * @throws InstantiationException
	 */
	public static void main(String[] args) throws Exception {
		XMLStreamWriter xmlWriter;
		FileOutputStream os;
		Event error;

		try {
			JobStarterDebug.main(args);
		} catch (Exception e) {
			System.err.println(e.getMessage());
			e.printStackTrace();

			error = new Event(EventType.ERROR, "", e.getMessage(), e.getStackTrace());

			os = new FileOutputStream("Error.xml");
			xmlWriter = XMLOutputFactory.newInstance().createXMLStreamWriter(os, "utf-8");
			xmlWriter.writeStartDocument("utf-8", "1.0");
			xmlWriter.writeComment(Project.getVersion());
			error.obj2xml(xmlWriter);
			xmlWriter.writeEndDocument();
			xmlWriter.close();
		}

		System.exit(0);
	}
}
